<?php

namespace FM\Controller;

use Zeedhi\Framework\DTO\Request;
use FM\Service\ParametrizacaoService;
use FM\Traits\TResponse;

class ParametrizacaoController
{
    use TResponse;
    protected $ParametrizacaoService;

    public function __construct(ParametrizacaoService $ParametrizacaoService)
    {
        $this->ParametrizacaoService = $ParametrizacaoService;
    }

    /*
     * TAB CATEGORIA DE MANUTENÇÃO
     */

    public function saveCategoriaManutencao(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->saveCategoriaManutencao($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCategoriasManutencao(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getCategoriasManutencao();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTecnicosRegistrados(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getTecnicosRegistrados();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteCategoriasManutencao(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->deleteCategoriasManutencao($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getAbrevCategoria(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getAbrevCategoria();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /*
     * TAB TIPO DE MANUTENÇÃO
     */

    public function saveTipoManutencao(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->saveTipoManutencao($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTiposManutencao(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getTiposManutencao();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteTiposManutencao(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->deleteTiposManutencao($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /*
     * TAB PRIORIDADES
     */

    public function savePrioridade(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->savePrioridade($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getPrioridadesManutencao()
    {
        try {
            $data = $this->ParametrizacaoService->getPrioridadesManutencao();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getPrioridades() 
    {
        try {
            $data = $this->ParametrizacaoService->getPrioridades();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getImpactosRelacionados()
    {
        try {
            $data = $this->ParametrizacaoService->getImpactosRelacionados();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deletePrioridades(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->deletePrioridades($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /*
     * TAB IMPACTO
     */

    public function saveImpacto(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->saveImpacto($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getImpactosManutencao(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getImpactosManutencao();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteImpactos(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->deleteImpactos($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getImpactos(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getImpactos();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /*
     * TAB ESCOPO
     */

    public function saveEscopo(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->saveEscopo($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getEscoposManutencao(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getEscoposManutencao();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteEscopos(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->deleteEscopos($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /*
     * TAB UNIDADE DE MEDIDA
     */

    public function saveUnidadeMedida(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->saveUnidadeMedida($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getUnidadesMedida(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getUnidadesMedida();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteUnidadesMedida(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->ParametrizacaoService->deleteUnidadesMedida($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getAbrevUniMedida(Request $request)
    {
        try {
            $data = $this->ParametrizacaoService->getAbrevUniMedida();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
