<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class EquipamentoFactory {
	use TQuery;

	protected $connection;
	protected $entityManager;

	public function __construct(EntityManager $entityManager) {
		$this->connection = $entityManager->getConnection();
		$this->entityManager = $entityManager;
	}
	
	public function getAtivos() {
		$sql = "SELECT DISTINCT FCM_EQUIPAMENTO.NREQUIPAMENTO, FCM_EQUIPAMENTO.CDEQUIPAMENTO, FCM_EQUIPAMENTO.NMEQUIPAMENTO, FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP,
					FCM_CATEGORIAEQUIP.NMCATEGORIAEQUIP, NVL((SELECT LISTAGG(FCM_TIPOMANUTENCAO.NMTIPOMANUTENCAO, ', ') WITHIN GROUP (ORDER BY FCM_TIPOMANUTENCAO.NMTIPOMANUTENCAO)
															FROM FCM_TIPOMANUTENCAO
														   	WHERE FCM_TIPOMANUTENCAO.NRTIPOMANUTENCAO IN (SELECT FCM_RELINSUMOSEQUIP.NRRELINSUMOSEQUIP
																										   	FROM FCM_RELINSUMOSEQUIP
																										  	WHERE FCM_RELINSUMOSEQUIP.NREQUIPAMENTO = FCM_EQUIPAMENTO.NREQUIPAMENTO)), ' ') as TIPOSMANUTENCAO,
					FCM_EQUIPAMENTO.IDSITUACAO AS IDSITUACAO,
					CASE FCM_EQUIPAMENTO.IDSITUACAO WHEN '1' THEN 'Em Funcionamento' WHEN '2' THEN 'Em Manutenção' WHEN '3' THEN 'Descontinuado' END as SITUACAO,
					CASE WHEN FCM_EQUIPAMENTO.DTULTATU IS NULL THEN ' ' ELSE TO_CHAR(FCM_EQUIPAMENTO.DTULTATU, 'DD/MM/YYYY') END AS DTULTATU
				FROM FCM_EQUIPAMENTO
				JOIN FCM_CATEGORIAEQUIP
				  ON FCM_EQUIPAMENTO.NRCATEGORIAEQUIP = FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
	}

	public function insertAtivo($parametros) {
		$sql = "INSERT INTO FCM_EQUIPAMENTO (
					NREQUIPAMENTO,		NMEQUIPAMENTO,	CDEQUIPAMENTO,	NRCATEGORIAEQUIP,
					IDSITUACAO,			NRORG,			IDATIVO,		DTINCLUSAO,
					CDOPERINCLUSAO,		NRORGINCLUSAO	
				)
				VALUES (
					:NREQUIPAMENTO,		:NMEQUIPAMENTO, :CDEQUIPAMENTO, :NRCATEGORIAEQUIP,
					:IDSITUACAO,		:NRORG,			'S',			SYSDATE,
					:CDOPERINCLUSAO,	:NRORGINCLUSAO
				)";
		
		$this->connection->executeStatement($sql, $parametros);
	}

	public function updateAtivo($parametros) {
		$sql = "UPDATE FCM_EQUIPAMENTO
				SET	NMEQUIPAMENTO	 = :NMEQUIPAMENTO,
					CDEQUIPAMENTO	 = :CDEQUIPAMENTO,
					NRCATEGORIAEQUIP = :NRCATEGORIAEQUIP,
					IDSITUACAO		 = :IDSITUACAO,
					DTULTATU		 = SYSDATE,
					CDOPERULTATU	 = :CDOPERULTATU,
					NRORGULTATU		 = :NRORGULTATU
				WHERE NREQUIPAMENTO = :NREQUIPAMENTO";

		$this->connection->executeStatement($sql, $parametros);
	}

	public function getNumRelacEquipEscopo($parametros) {
		$sql = "SELECT COUNT(*)
				FROM FCM_EQUIPAMENTO
				WHERE NREQUIPAMENTO = :NREQUIPAMENTO
				AND	NRESCOPOMANUT = :NRESCOPOMANUT";

		return $this->connection->fetchOne($sql, $parametros);
	}

	public function deletRelacEscopoEquip($parametros){
        $sql = "DELETE FROM FCM_RELACESCOPOSEQUIP
               	WHERE NREQUIPAMENTO = :NREQUIPAMENTO";
        
		$this->connection->executeStatement($sql, $parametros);
    }
    
    public function insertRelacEscopoEquip($parametros){
        $sql = "INSERT INTO FCM_RELACESCOPOSEQUIP (RELACESCOPOSEQUIP, NREQUIPAMENTO, NRESCOPOMANUT, NRORG, IDATIVO, DTINCLUSAO)
              	VALUES (:RELACESCOPOSEQUIP, :NREQUIPAMENTO, :NRESCOPOMANUT, :NRORG, 'S', SYSDATE)";
        
		$this->connection->executeStatement($sql, $parametros);
    }

	public function insertImgAtivo($parametros){
        $sql = "INSERT INTO FCM_IMGEQUIPAMENTO (NRIMGEQUIPAMENTO, NREQUIPAMENTO, DSCAMINHO, NRORG, IDATIVO, DSTYPE, NRSIZE)
              	VALUES (:NRIMGEQUIPAMENTO, :NREQUIPAMENTO, :DSCAMINHO, :NRORG, 'S', :DSTYPE, :NRSIZE)";
        
        $this->connection->executeStatement($sql, $parametros);
    }

	public function getFotosAtivo($parametros) {
		$sql = "SELECT NRIMGEQUIPAMENTO, DSCAMINHO
				FROM FCM_IMGEQUIPAMENTO
				WHERE NREQUIPAMENTO = :NREQUIPAMENTO";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
	}

	public function deleteFotoAtivo($parametros) {
		$sql = "DELETE
				FROM FCM_IMGEQUIPAMENTO
				WHERE NRIMGEQUIPAMENTO = :NRIMGEQUIPAMENTO";
		
		$this->connection->executeStatement($sql, $parametros);
	}

	public function deleteAtivos($parametros) {
		$sql = "DELETE 
				FROM FCM_EQUIPAMENTO
				WHERE NREQUIPAMENTO IN (".$parametros.")";

		$this->connection->executeStatement($sql);
	}

	public function getCategEquip() {
		$sql = "SELECT NRCATEGORIAEQUIP, NMCATEGORIAEQUIP
				FROM FCM_CATEGORIAEQUIP";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
	}

	public function getEscopos() {
		$sql = "SELECT NRESCOPOMANUT, NMESCOPOMANUT
				FROM FCM_ESCOPOMANUT";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
	}
}

?>