<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class ParametrizacaoFactory {
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /*
     * TAB CATEGORIA DE MANUTENÇÃO
     */

     public function insertCategoriaManutencao($parametros) {
        $sql = "INSERT INTO FCM_CATEGORIAMANUT (
                    NRCATEGORIAMANUT,   NMCATEGORIAMANUT,   NMABREVCATEGMANUT,  NRORG,
                    IDATIVO,            DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,
                    DTULTATU,           CDOPERULTATU,       NRORGULTATU
                )
                VALUES (
                    :NRCATEGORIAMANUT,  :NMCATEGORIAMANUT,  :NMABREVCATEGMANUT, :NRORG,
                    'S',                SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,
                    SYSDATE,            :CDOPERULTATU,      :NRORGULTATU
                )";
        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function editCategoriaManutencao($parametros) {
        $sql = "UPDATE FCM_CATEGORIAMANUT
                SET NMCATEGORIAMANUT    = :NMCATEGORIAMANUT,
                    NMABREVCATEGMANUT   = :NMABREVCATEGMANUT,
                    DTULTATU            = SYSDATE,
                    CDOPERULTATU        = :CDOPERULTATU,
                    NRORGULTATU         = :NRORGULTATU
                WHERE NRCATEGORIAMANUT = :NRCATEGORIAMANUT";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getCategoriasManutencao() {

        $sql = "SELECT FCM_CATEGORIAMANUT.NRCATEGORIAMANUT, FCM_CATEGORIAMANUT.NMABREVCATEGMANUT, FCM_CATEGORIAMANUT.NMCATEGORIAMANUT,
                  (SELECT LISTAGG(FCM_FUNCIONARIO.NMFUNCIONARIO, ',') 
                     WITHIN GROUP (ORDER BY FCM_FUNCCATEGORIAMANUT.NRFUNCIONARIO)
                             FROM FCM_FUNCCATEGORIAMANUT
                             JOIN FCM_FUNCIONARIO
                               ON FCM_FUNCCATEGORIAMANUT.NRFUNCIONARIO = FCM_FUNCIONARIO.NRFUNCIONARIO
                            WHERE FCM_CATEGORIAMANUT.NRCATEGORIAMANUT = FCM_FUNCCATEGORIAMANUT.NRCATEGORIAMANUT) NRFUNCIONARIO
                    FROM FCM_CATEGORIAMANUT
                ORDER BY FCM_CATEGORIAMANUT.NRCATEGORIAMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTecnicosRegistrados() {
        $sql = "SELECT DISTINCT FUNCAT.NRFUNCIONARIO, FUNC.NMFUNCIONARIO
                    FROM FCM_FUNCCATEGORIAMANUT FUNCAT
                    JOIN FCM_FUNCIONARIO FUNC
                      ON FUNC.NRFUNCIONARIO = FUNCAT.NRFUNCIONARIO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteCategoriasManutencao($parametros) {
        $sql = "DELETE
                FROM FCM_CATEGORIAMANUT
                WHERE NRCATEGORIAMANUT IN (".$parametros.")";

        $this->connection->executeStatement($sql);
    }

    public function getAbrevCategoria() {
        $sql = "SELECT NMABREVCATEGMANUT
                FROM FCM_CATEGORIAMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    /*
     * TAB TIPO DE MANUTENÇÃO
     */

     public function insertTipoManutencao($parametros) {

        $sql = "INSERT INTO FCM_TIPOMANUTENCAO (
                    NRTIPOMANUTENCAO,   NMTIPOMANUTENCAO,   NRORG,          IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,  DTULTATU,
                    CDOPERULTATU,       NRORGULTATU
                )
                VALUES (
                    :NRTIPOMANUTENCAO,  :NMTIPOMANUTENCAO,  :NRORG,         'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO, SYSDATE,
                    :CDOPERULTATU,      :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function editTipoManutencao($parametros) {
        $sql = "UPDATE FCM_TIPOMANUTENCAO
                SET NMTIPOMANUTENCAO    = :NMTIPOMANUTENCAO,
                    DTULTATU            = SYSDATE,
                    CDOPERULTATU        = :CDOPERULTATU,
                    NRORGULTATU         = :NRORGULTATU
                WHERE NRTIPOMANUTENCAO  = :NRTIPOMANUTENCAO";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getTiposManutencao() {

        $sql = "SELECT NRTIPOMANUTENCAO, NMTIPOMANUTENCAO
                FROM FCM_TIPOMANUTENCAO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteTiposManutencao($parametros) {
        $sql = "DELETE
                FROM FCM_TIPOMANUTENCAO
                WHERE NRTIPOMANUTENCAO IN (".$parametros.")";

        $this->connection->executeStatement($sql);
    }

    /*
     * TAB PRIORIDADES
     */

    public function insertPrioridade($parametros)
    {

        $sql = "INSERT INTO FCM_PRIORIDADEMANUT (
                    NRPRIORIDADEMANUT,  NMPRIORIDADEMANUT,  DSCORPRIORIDADE,    NRORG,      IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,      DTULTATU,   CDOPERULTATU, 
                    NRORGULTATU
                )
                VALUES (
                    :NRPRIORIDADEMANUT, :NMPRIORIDADEMANUT, :DSCORPRIORIDADE,   :NRORG,     'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,     SYSDATE,    :CDOPERULTATU,
                    :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function editPrioridade($parametros)
    {
        $sql = "UPDATE FCM_PRIORIDADEMANUT
                SET NMPRIORIDADEMANUT   = :NMPRIORIDADEMANUT,
                    DSCORPRIORIDADE     = :DSCORPRIORIDADE,
                    DTULTATU            = SYSDATE,
                    CDOPERULTATU        = :CDOPERULTATU,
                    NRORGULTATU         = :NRORGULTATU
                WHERE NRPRIORIDADEMANUT = :NRPRIORIDADEMANUT";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function verificaRelPrioridadeImpacto($parametros)
    {
        $sql = "SELECT COUNT(*)
                FROM FCM_RELPRIORIDADEIMPACTO
                WHERE NRPRIORIDADEMANUT = :NRPRIORIDADEMANUT AND NRIMPACTOMANUT = :NRIMPACTOMANUT";

        return $this->connection->fetchOne($sql, $parametros);
    }

    public function deleteImpactosDesatualizados($parametros, $impactos)
    {
        $sql = "DELETE
                FROM FCM_RELPRIORIDADEIMPACTO
                WHERE NRPRIORIDADEMANUT = :NRPRIORIDADEMANUT AND NRIMPACTOMANUT NOT IN (" . $impactos . ")";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updatePrioridadePadrao($parametros, $impactos)
    {
        $sql = "UPDATE FCM_IMPACTOMANUT
                SET NRPRIORIDADEMANUTPADR = NULL
                WHERE NRPRIORIDADEMANUTPADR = :NRPRIORIDADEMANUT AND NRIMPACTOMANUT NOT IN (" . $impactos .")";
                
        $this->connection->executeStatement($sql, $parametros);
    }

    public function getPrioridadesManutencao()
    {
        $sql = "SELECT P.NRPRIORIDADEMANUT, P.NMPRIORIDADEMANUT, P.DSCORPRIORIDADE,
                    (SELECT LISTAGG(FCM_IMPACTOMANUT.NRIMPACTOMANUT, ',') WITHIN GROUP (ORDER BY FCM_RELPRIORIDADEIMPACTO.NRIMPACTOMANUT)
                            FROM FCM_RELPRIORIDADEIMPACTO
                            JOIN FCM_IMPACTOMANUT
                              ON FCM_RELPRIORIDADEIMPACTO.NRIMPACTOMANUT = FCM_IMPACTOMANUT.NRIMPACTOMANUT
                           WHERE P.NRPRIORIDADEMANUT = FCM_RELPRIORIDADEIMPACTO.NRPRIORIDADEMANUT) NRIMPACTOMANUT
                        FROM FCM_PRIORIDADEMANUT P
                    ORDER BY P.NRPRIORIDADEMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, [], [], null, null, null, array("NRIMPACTOMANUT"));
    }

    public function getPrioridades()
    {
        $sql = "SELECT NRPRIORIDADEMANUT, NMPRIORIDADEMANUT
                FROM FCM_PRIORIDADEMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getImpactosRelacionados()
    {
        $sql = "SELECT NRIMPACTOMANUT, NMIMPACTOMANUT
                FROM FCM_IMPACTOMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function verificaRelOrdemServ($parametros)
    {
        $sql = "SELECT COUNT(*)
                FROM FCM_ORDEMSERV
                WHERE NRPRIORIDADEMANUT IN (" . $parametros .")";

        return $this->connection->fetchOne($sql);
    }

    public function verificaRelSolicitServ($parametros)
    {
        $sql = "SELECT COUNT(*)
                FROM FCM_SOLICITACAOSERV
                WHERE NRPRIORIDADEMANUT IN (" . $parametros .")";

        return $this->connection->fetchOne($sql);
    }

    public function deletePrioridades($parametros)
    {
        $sql = "DELETE
                FROM FCM_PRIORIDADEMANUT
                WHERE NRPRIORIDADEMANUT IN (" . $parametros . ")";

        $this->connection->executeStatement($sql);
    }

    public function deleteImpactosRelacionados($parametros)
    {
        $sql = "DELETE
                FROM FCM_RELPRIORIDADEIMPACTO
                WHERE NRPRIORIDADEMANUT IN (" . $parametros . ")";

        $this->connection->executeStatement($sql);
    }

    /*
     * TAB IMPACTO
     */

     public function insertImpacto($parametros) {
        $sql = "INSERT INTO FCM_IMPACTOMANUT (
                    NRIMPACTOMANUT,     NMIMPACTOMANUT,     NRPRIORIDADEMANUTPADR,  NRORG,      IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,          DTULTATU,   CDOPERULTATU, 
                    NRORGULTATU
                )
                VALUES (
                    :NRIMPACTOMANUT,    :NMIMPACTOMANUT,    :NRPRIORIDADEMANUTPADR, :NRORG,     'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,         SYSDATE,    :CDOPERULTATU,
                    :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function editImpacto($parametros) {
        $sql = "UPDATE FCM_IMPACTOMANUT
                SET NMIMPACTOMANUT          = :NMIMPACTOMANUT,
                    NRPRIORIDADEMANUTPADR   = :NRPRIORIDADEMANUTPADR,
                    DTULTATU                = SYSDATE,
                    CDOPERULTATU            = :CDOPERULTATU,
                    NRORGULTATU             = :NRORGULTATU
                WHERE NRIMPACTOMANUT = :NRIMPACTOMANUT";

        $this->connection->executeStatement($sql, $parametros);
    }
    
    public function getImpactosManutencao() {
        
        $sql = "SELECT DISTINCT I.NRIMPACTOMANUT, I.NMIMPACTOMANUT, I.NRPRIORIDADEMANUTPADR, P.NMPRIORIDADEMANUT
                    FROM FCM_IMPACTOMANUT I
                    LEFT JOIN FCM_PRIORIDADEMANUT P
                    ON P.NRPRIORIDADEMANUT = I.NRPRIORIDADEMANUTPADR
                    LEFT JOIN FCM_RELPRIORIDADEIMPACTO R
                    ON R.NRIMPACTOMANUT = I.NRIMPACTOMANUT";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteImpactos($parametros) {
        $sql = "DELETE
                FROM FCM_IMPACTOMANUT
                WHERE NRIMPACTOMANUT IN (".$parametros.")";

        $this->connection->executeStatement($sql);
    }

    public function verificaRelImpactoPrioridade($parametros) {
        $sql = "SELECT COUNT(*)
                FROM FCM_RELPRIORIDADEIMPACTO
                WHERE NRIMPACTOMANUT = :NRIMPACTOMANUT AND NRPRIORIDADEMANUT = :NRPRIORIDADEMANUT";

        return $this->connection->fetchOne($sql, $parametros);
    }

    public function insertImpactoRelacionado($parametros) {
        $sql = "INSERT INTO FCM_RELPRIORIDADEIMPACTO (
                    RELPRIORIDADEIMPACTO,   NRPRIORIDADEMANUT,  NRIMPACTOMANUT,     NRORG,
                    IDATIVO,                DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,
                    DTULTATU,               CDOPERULTATU,       NRORGULTATU
                )
                VALUES (
                    :RELPRIORIDADEIMPACTO,  :NRPRIORIDADEMANUT, :NRIMPACTOMANUT,    :NRORG,
                    'S',                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,
                    SYSDATE,                :CDOPERULTATU,      :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getImpactos() {
        $sql = "SELECT NRIMPACTOMANUT, NMIMPACTOMANUT
                FROM FCM_IMPACTOMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    /*
     * TAB ESCOPO
     */

     public function insertEscopo($parametros) {

        $sql = "INSERT INTO FCM_ESCOPOMANUT (
                    NRESCOPOMANUT,  NMESCOPOMANUT,      NRORG,          IDATIVO,
                    DTINCLUSAO,     CDOPERINCLUSAO,     NRORGINCLUSAO,  DTULTATU,
                    CDOPERULTATU,   NRORGULTATU
                )
                VALUES (
                    :NRESCOPOMANUT, :NMESCOPOMANUT,     :NRORG,         'S',
                    SYSDATE,        :CDOPERINCLUSAO,    :NRORGINCLUSAO, SYSDATE,
                    :CDOPERULTATU,  :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function editEscopo($parametros) {
        $sql = "UPDATE FCM_ESCOPOMANUT
                SET NMESCOPOMANUT   = :NMESCOPOMANUT,
                    DTULTATU        = SYSDATE,
                    CDOPERULTATU    = :CDOPERULTATU,
                    NRORGULTATU     = :NRORGULTATU
                WHERE NRESCOPOMANUT = :NRESCOPOMANUT";
        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function getEscoposManutencao() {

        $sql = "SELECT NRESCOPOMANUT, NMESCOPOMANUT
                FROM FCM_ESCOPOMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteEscopos($parametro) {

        $sql = "DELETE 
                FROM FCM_ESCOPOMANUT
                WHERE NRESCOPOMANUT IN (".$parametro.")";

        $this->connection->executeStatement($sql);
    }

    /*
     * TAB UNIDADE DE MEDIDA
     */

     public function insertUnidadeMedida($parametros) {
        $sql = "INSERT INTO FCM_UNIDADEMEDIDA (
                    NRUNIDADEMEDIDA,    NMUNIDADEMEDIDA,    NMABREVUNIDADEMEDIDA,   NRORG,      IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,          DTULTATU,   CDOPERULTATU, 
                    NRORGULTATU
                )
                VALUES (
                    :NRUNIDADEMEDIDA,   :NMUNIDADEMEDIDA,   :NMABREVUNIDADEMEDIDA,  :NRORG,     'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,         SYSDATE,    :CDOPERULTATU,
                    :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function editUnidadeMedida($parametros) {
        $sql = "UPDATE FCM_UNIDADEMEDIDA
                SET NMUNIDADEMEDIDA         = :NMUNIDADEMEDIDA,
                    NMABREVUNIDADEMEDIDA    = :NMABREVUNIDADEMEDIDA,
                    DTULTATU                = SYSDATE,
                    CDOPERULTATU            = :CDOPERULTATU,
                    NRORGULTATU             = :NRORGULTATU
                WHERE NRUNIDADEMEDIDA = :NRUNIDADEMEDIDA";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getUnidadesMedida() {
        $sql = "SELECT NRUNIDADEMEDIDA, NMUNIDADEMEDIDA, NMABREVUNIDADEMEDIDA
                FROM FCM_UNIDADEMEDIDA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteUnidadesMedida($parametro) {
        $sql = "DELETE 
                FROM FCM_UNIDADEMEDIDA
                WHERE NRUNIDADEMEDIDA IN (".$parametro.")";

        $this->connection->executeStatement($sql);
    }

    public function getAbrevUniMedida() {
        $sql = "SELECT NMABREVUNIDADEMEDIDA
                FROM FCM_UNIDADEMEDIDA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}
