<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class PrioridadeManutFactory
{
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function inserirPrioridade($parametros)
    {

        $sql = "INSERT INTO FCM_PRIORIDADEMANUT(NRPRIORIDADEMANUT, NMPRIORIDADEMANUT, DSCORPRIORIDADE, DTINCLUSAO)
                VALUES (:NRPRIORIDADEMANUT, :NMPRIORIDADEMANUT, :DSCORPRIORIDADE, SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertPrioridade($parametros)
    {

        $sql = "INSERT INTO FCM_PRIORIDADEMANUT (
                    NRPRIORIDADEMANUT,  NMPRIORIDADEMANUT,  DSCORPRIORIDADE,    NRORG,      IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,      DTULTATU,   CDOPERULTATU, 
                    NRORGULTATU
                )
                VALUES (
                    :NRPRIORIDADEMANUT, :NMPRIORIDADEMANUT, :DSCORPRIORIDADE,   :NRORG,     'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,     SYSDATE,    :CDOPERULTATU,
                    :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function editPrioridade($parametros)
    {
        $sql = "UPDATE FCM_PRIORIDADEMANUT
                SET NMPRIORIDADEMANUT   = :NMPRIORIDADEMANUT,
                    DSCORPRIORIDADE     = :DSCORPRIORIDADE,
                    DTULTATU            = SYSDATE,
                    CDOPERULTATU        = :CDOPERULTATU,
                    NRORGULTATU         = :NRORGULTATU
                WHERE NRPRIORIDADEMANUT = :NRPRIORIDADEMANUT";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function verificaRelPrioridadeImpacto($parametros)
    {
        $sql = "SELECT COUNT(*)
                FROM FCM_RELPRIORIDADEIMPACTO
                WHERE NRPRIORIDADEMANUT = :NRPRIORIDADEMANUT AND NRIMPACTOMANUT = :NRIMPACTOMANUT";

        return $this->connection->fetchOne($sql, $parametros);
    }

    public function deleteImpactosDesatualizados($parametros, $impactos)
    {
        $sql = "DELETE
                FROM FCM_RELPRIORIDADEIMPACTO
                WHERE NRPRIORIDADEMANUT = :NRPRIORIDADEMANUT AND NRIMPACTOMANUT NOT IN (" . $impactos . ")";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updatePrioridadePadrao($parametros, $impactos)
    {
        $sql = "UPDATE FCM_IMPACTOMANUT
                SET NRPRIORIDADEMANUTPADR = NULL
                WHERE NRPRIORIDADEMANUTPADR = :NRPRIORIDADEMANUT AND NRIMPACTOMANUT NOT IN (" . $impactos .")";
                
        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertImpactoRelacionado($parametros)
    {
        $sql = "INSERT INTO FCM_RELPRIORIDADEIMPACTO (
                    RELPRIORIDADEIMPACTO,   NRPRIORIDADEMANUT,  NRIMPACTOMANUT,     NRORG,
                    IDATIVO,                DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,
                    DTULTATU,               CDOPERULTATU,       NRORGULTATU
                )
                VALUES (
                    :RELPRIORIDADEIMPACTO,  :NRPRIORIDADEMANUT, :NRIMPACTOMANUT,    :NRORG,
                    'S',                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,
                    SYSDATE,                :CDOPERULTATU,      :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getPrioridadesManutencao()
    {

        $sql = "SELECT P.NRPRIORIDADEMANUT, P.NMPRIORIDADEMANUT, P.DSCORPRIORIDADE,
                    (SELECT LISTAGG(FCM_IMPACTOMANUT.NRIMPACTOMANUT, ',') WITHIN GROUP (ORDER BY FCM_RELPRIORIDADEIMPACTO.NRIMPACTOMANUT)
                            FROM FCM_RELPRIORIDADEIMPACTO
                            JOIN FCM_IMPACTOMANUT
                              ON FCM_RELPRIORIDADEIMPACTO.NRIMPACTOMANUT = FCM_IMPACTOMANUT.NRIMPACTOMANUT
                           WHERE P.NRPRIORIDADEMANUT = FCM_RELPRIORIDADEIMPACTO.NRPRIORIDADEMANUT) NRIMPACTOMANUT
                        FROM FCM_PRIORIDADEMANUT P
                    ORDER BY P.NRPRIORIDADEMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, [], [], null, null, null, array("NRIMPACTOMANUT"));
    }

    public function getPrioridades()
    {
        $sql = "SELECT NRPRIORIDADEMANUT, NMPRIORIDADEMANUT
                FROM FCM_PRIORIDADEMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirImpactoRelacionado($parametros)
    {

        $sql = "INSERT INTO FCM_RELPRIORIDADEIMPACTO(RELPRIORIDADEIMPACTO, NRPRIORIDADEMANUT, NRIMPACTOMANUT, DTINCLUSAO)
                    VALUES(:RELPRIORIDADEIMPACTO, :NRPRIORIDADEMANUT, :NRIMPACTOMANUT, SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getImpactosRelacionados()
    {
        $sql = "SELECT NRIMPACTOMANUT, NMIMPACTOMANUT
                FROM FCM_IMPACTOMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deletePrioridades($parametros)
    {
        $sql = "DELETE
                FROM FCM_PRIORIDADEMANUT
                WHERE NRPRIORIDADEMANUT IN (" . $parametros . ")";

        $this->connection->executeStatement($sql);
    }

    public function deleteImpactosRelacionados($parametros)
    {
        $sql = "DELETE
                FROM FCM_RELPRIORIDADEIMPACTO
                WHERE NRPRIORIDADEMANUT IN (" . $parametros . ")";

        $this->connection->executeStatement($sql);
    }
}
