<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class SolicitacaoServFactory {
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getSolicitacoesServicos() {
        $sql = "SELECT solServ.NRSOLICITACAOSERV, TO_CHAR(solServ.DTSOLICITACAOSERV, 'DD/MM/YY') AS DTSOLICITACAOSERV, catManut.NMCATEGORIAMANUT, equip.NMEQUIPAMENTO,
                    solServ.DSSOLICITACAOSERV, NVL(oper.NMOPERADOR, '') as NMOPERADOR,
                    solServ.NRFUNCSOLICITANTE, solServ.IDSITUACAO,
                    solServ.NREQUIPAMENTO, solServ.NRCATEGORIAMANUT,
                    solServ.NRIMPACTOMANUT, impManut.NMIMPACTOMANUT, solServ.NRPRIORIDADEMANUT, solServ.NMSOLICITACAOSERV,
                    (SELECT LISTAGG(FCM_IMGSOLICITACAOSERV.DSCAMINHO, ', ') WITHIN GROUP (ORDER BY FCM_IMGSOLICITACAOSERV.DSCAMINHO)
                       FROM FCM_IMGSOLICITACAOSERV
                      WHERE solServ.NRSOLICITACAOSERV = FCM_IMGSOLICITACAOSERV.NRSOLICITACAOSERV) AS DSCAMINHOABC,
                    (SELECT LISTAGG(escManut.NRESCOPOMANUT, ', ') WITHIN GROUP (ORDER BY escManut.NRESCOPOMANUT)
                       FROM FCM_RELESCSOLICITASERV relEscoSol
                       JOIN FCM_ESCOPOMANUT escManut
                         ON relEscoSol.NRESCOPOMANUT = escManut.NRESCOPOMANUT
                      WHERE solServ.NRSOLICITACAOSERV = relEscoSol.NRSOLICITACAOSERV) AS NRESCOPOMANUT,
                    (SELECT LISTAGG(escManut.NMESCOPOMANUT, ', ') WITHIN GROUP (ORDER BY escManut.NRESCOPOMANUT)
                       FROM FCM_RELESCSOLICITASERV relEscoSol
                       JOIN FCM_ESCOPOMANUT escManut
                         ON relEscoSol.NRESCOPOMANUT = escManut.NRESCOPOMANUT
                      WHERE solServ.NRSOLICITACAOSERV = relEscoSol.NRSOLICITACAOSERV) AS NMESCOPOMANUT,
                    CASE solServ.IDSITUACAO
                        WHEN '1' THEN 'Pendente'
                        WHEN '2' THEN 'Em Andamento'
                        WHEN '3' THEN 'Aguardando vistoria'
                        WHEN '4' THEN 'Concluída'
                        WHEN '5' THEN 'Cancelada'
                        ELSE NULL
                    END AS SITUACAO
                FROM FCM_SOLICITACAOSERV solServ
                INNER JOIN FCM_EQUIPAMENTO equip
                    ON solServ.NREQUIPAMENTO = equip.NREQUIPAMENTO
                INNER JOIN FCM_CATEGORIAMANUT catManut
                    ON solServ.NRCATEGORIAMANUT = catManut.NRCATEGORIAMANUT
                LEFT JOIN OPERADOR oper
                    ON solServ.CDOPERSOLICITANTE = oper.CDOPERADOR
                  JOIN FCM_IMPACTOMANUT impManut
                    ON solServ.NRIMPACTOMANUT = impManut.NRIMPACTOMANUT";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function recusarSolicitacao($parametros) {
        $sql = "UPDATE FCM_SOLICITACAOSERV
                SET IDSITUACAO = 5
                WHERE NRSOLICITACAOSERV = :NRSOLICITACAOSERV";
        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertComentarioSolicitacao($parametros) {
        $sql = "INSERT INTO FCM_COMENTSOLICITSERV (
                    NRCOMENTSOLICITSERV,    NRSOLICITACAOSERV,  DTCOMENTARIO,       DSCOMENTARIO,
                    NRORG,                  DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,
                    DTULTATU,               CDOPERULTATU,       NRORGULTATU,        CDOPERCOMENTARIO
                )
                VALUES (
                    :NRCOMENTSOLICITSERV,   :NRSOLICITACAOSERV, SYSDATE,            :DSCOMENTARIO,
                    :NRORG,                 SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,
                    SYSDATE,                :CDOPERULTATU,      :NRORGULTATU,       :CDOPERCOMENTARIO
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function saveSolicitacaoServico($parametros) {
        $sql = "INSERT INTO FCM_SOLICITACAOSERV (
                    NRSOLICITACAOSERV,  DTSOLICITACAOSERV,  NREQUIPAMENTO,      NRCATEGORIAMANUT,
                    NRIMPACTOMANUT,     DSSOLICITACAOSERV,  NRORG,              IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,      DTULTATU,
                    CDOPERULTATU,       NRORGULTATU,        NMSOLICITACAOSERV,  CDOPERSOLICITANTE
                )
                VALUES (
                    :NRSOLICITACAOSERV, SYSDATE,            :NREQUIPAMENTO,     :NRCATEGORIAMANUT,
                    :NRIMPACTOMANUT,    :DSSOLICITACAOSERV, :NRORG,             'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,     SYSDATE,
                    :CDOPERULTATU,      :NRORGULTATU,       :NMSOLICITACAOSERV, :CDOPERSOLICITANTE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertFotoSolicitacao($parametros) {
        $sql = "INSERT INTO FCM_IMGSOLICITACAOSERV (
                    NRIMGSOLICITACAOSERV,   NRSOLICITACAOSERV,  DSCAMINHO,  NRORG,
                    DTINCLUSAO,             DSTYPE,             NRSIZE
                )
                VALUES (
                    :NRIMGSOLICITACAOSERV,  :NRSOLICITACAOSERV, :DSCAMINHO, :NRORG,
                    SYSDATE,                :DSTYPE,            :NRSIZE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getImpactoManut() {
        $sql = "SELECT NRIMPACTOMANUT, NMIMPACTOMANUT
                FROM FCM_IMPACTOMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getPrioridadesManut() {
        $sql = "SELECT NRPRIORIDADEMANUT, NMPRIORIDADEMANUT
                FROM FCM_PRIORIDADEMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFotosSolicitacao($parametros) {
        $sql = "SELECT NRIMGSOLICITACAOSERV, DSCAMINHO
                FROM FCM_IMGSOLICITACAOSERV
                WHERE NRSOLICITACAOSERV= :NRSOLICITACAOSERV";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getComentariosSolicitacao($parametros) {
        $sql = "SELECT COMENT.NRCOMENTSOLICITSERV, NVL(OPER.NMOPERADOR, 'Anônimo') AS NMOPERADOR, TO_CHAR(COMENT.DTINCLUSAO, 'DD/MM/YYYY') AS DTINCLUSAO, COMENT.DSCOMENTARIO
                FROM FCM_COMENTSOLICITSERV COMENT
                LEFT JOIN OPERADOR OPER
                ON COMENT.CDOPERCOMENTARIO = OPER.CDOPERADOR
                WHERE COMENT.NRSOLICITACAOSERV = :NRSOLICITACAOSERV
                ORDER BY DTINCLUSAO, NRCOMENTSOLICITSERV DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getOperadores() {
        $sql = "SELECT CDOPERADOR, NMOPERADOR
                FROM OPERADOR
                WHERE IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFuncionarios() {
        $sql = "SELECT NRFUNCIONARIO, NMFUNCIONARIO
                FROM FCM_FUNCIONARIO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFuncionarioLogado($parametros) {
        $sql = "SELECT NRFUNCIONARIO
                FROM FCM_FUNCIONARIO
                WHERE CDOPERADOR = :CDOPERADOR";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function listSolicitacaoServ($params) {
        $sql = "SELECT NRSOLICITACAOSERV, DTSOLICITACAOSERV, NRFUNCSOLICITANTE,
                    IDSITUACAO, NREQUIPAMENTO, NRCATEGORIAMANUT,
                    NRIMPACTOMANUT, NRPRIORIDADEMANUT, DSSOLICITACAOSERV,
                    NMSOLICITACAOSERV
                FROM FCM_SOLICITACAOSERV";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }

    public function listSolicitacaoServJoin($params) {
        $sql = "SELECT solServ.NRSOLICITACAOSERV, TO_CHAR(solServ.DTSOLICITACAOSERV, 'DD/MM/YY') AS DTSOLICITACAOSERV, catManut.NMCATEGORIAMANUT, equip.NMEQUIPAMENTO,
                    solServ.DSSOLICITACAOSERV, func.NMFUNCIONARIO,
                    solServ.NRFUNCSOLICITANTE, solServ.IDSITUACAO,
                    solServ.NREQUIPAMENTO, solServ.NRCATEGORIAMANUT,
                    solServ.NRIMPACTOMANUT, impManut.NMIMPACTOMANUT, solServ.NRPRIORIDADEMANUT, solServ.NMSOLICITACAOSERV,
                    (SELECT LISTAGG(FCM_IMGSOLICITACAOSERV.DSCAMINHO, ', ') WITHIN GROUP (ORDER BY FCM_IMGSOLICITACAOSERV.DSCAMINHO)
                       FROM FCM_IMGSOLICITACAOSERV
                      WHERE solServ.NRSOLICITACAOSERV = FCM_IMGSOLICITACAOSERV.NRSOLICITACAOSERV) AS DSCAMINHO,
                    (SELECT LISTAGG(escManut.NRESCOPOMANUT, ', ') WITHIN GROUP (ORDER BY escManut.NRESCOPOMANUT)
                       FROM FCM_RELESCSOLICITASERV relEscoSol
                       JOIN FCM_ESCOPOMANUT escManut
                         ON relEscoSol.NRESCOPOMANUT = escManut.NRESCOPOMANUT
                      WHERE solServ.NRSOLICITACAOSERV = relEscoSol.NRSOLICITACAOSERV) AS NRESCOPOMANUT,
                    CASE solServ.IDSITUACAO
                        WHEN '1' THEN 'Pendente'
                        WHEN '2' THEN 'Em Andamento'
                        WHEN '3' THEN 'Aguardando vistoria'
                        WHEN '4' THEN 'Concluída'
                        ELSE NULL
                    END AS IDSITUACAO
                FROM FCM_SOLICITACAOSERV solServ
                INNER JOIN FCM_EQUIPAMENTO equip
                    ON solServ.NREQUIPAMENTO = equip.NREQUIPAMENTO
                INNER JOIN FCM_CATEGORIAMANUT catManut
                    ON solServ.NRCATEGORIAMANUT = catManut.NRCATEGORIAMANUT
                INNER JOIN FCM_FUNCIONARIO func
                    ON solServ.NRFUNCSOLICITANTE = func.NRFUNCIONARIO
                  JOIN FCM_IMPACTOMANUT impManut
                    ON solServ.NRIMPACTOMANUT = impManut.NRIMPACTOMANUT";
        $whereClausule = $this->concatWhereClausule($params);
        $sql .= $whereClausule;
        $std = $this->connection->prepare($sql);
        if (strlen($whereClausule) > 0) {
            $std->execute($params);
        } else {
            $std->execute();
        }
        return $std->fetchAll();
    }

    protected function concatWhereClausule($params) {
        $useParams = false;
        $paramKeys = array_keys($params);
        $query = "";
        $mainTable = "solServ";
        foreach($paramKeys as $key) {
            if (!$useParams) {
                $useParams = true;
                $query.= " WHERE ";
            } else {
                $query.= " AND ";
            }
            $query.= $mainTable . "." . $key . " = :" . $key;
        }
        return $query;
    }
}

?>