<?php

namespace FM\Service;

use FM\Factory\OrdemServFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FileServerService;


class OrdemServService {

    protected $connection;
    protected $OrdemServFactory;
    protected $Environment;
    protected $newCode;
    protected $FileServerService;
    protected $fileServerUrl;

    public function __construct (EntityManager $entityManager, OrdemServFactory $OrdemServFactory, Environment $Environment, NewCode $newCode, FileServerService $FileServerService, $fileServerUrl) {
        $this->OrdemServFactory  = $OrdemServFactory;
        $this->connection        = $entityManager->getConnection();
        $this->Environment       = $Environment;
        $this->newCode           = $newCode;
        $this->FileServerService = $FileServerService;
        $this->fileServerUrl     = $fileServerUrl;

    }

    public function getOrdemServ($NR){
        return $this->OrdemServFactory->getOrdemServ($NR);
    }

    // Função que buscas todas as ordens de serviço para montagem dos cards da tela de Ordens de Serviço
    public function getInformacoesOrdemServico() {
        return $this->OrdemServFactory->getInformacoesOrdemServico();
    }

    public function getOrdensServico()
    {
        return $this->OrdemServFactory->getOrdensServico();
    }

    public function getComentariosOrdem($parametros)
    {
        $parametroComentario = array(
            "NRORDEMSERV" => $parametros["NRORDEMSERV"]
        );

        return $this->OrdemServFactory->getComentariosOrdem($parametroComentario);
    }

    public function saveComentarioOrdem($parametros) {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $NRCOMENTORDEMSERV = $this->newCode->getCodeByTableName("FCM_COMENTORDEMSERV", $nrorg, 1, 10, true);

            $parametroComentario = array(
                "NRCOMENTORDEMSERV" => $NRCOMENTORDEMSERV,
                "NRORDEMSERV"       => $parametros["NRORDEMSERV"],
                "DSCOMENTARIO"      => $parametros["DSCOMENTARIO"]
            );

            $this->OrdemServFactory->saveComentarioOrdem($parametroComentario);

            $this->connection->commit();
        } catch (\Exception $e) {
            var_dump($e->getMessage());
            $this->connection->rollback();
        }
    }

    // Função para buscar os comentários associados à uma ordem de serviço
    public function comentariosOrdemServico($parametros) {

        $param = array(
            'NRORDEMSERV' => $parametros['NRORDEMSERV']);

        return $this->OrdemServFactory->comentariosOrdemServico($param);
    }

    // Função que consulta se a ordem de serviço especifica tem relação com uma ordem de serviço
    public function consultarRelacionamentoComSolicitacaoServico($parametros) {

        $param = array(
            'NRORDEMSERV' => $parametros['NRORDEMSERV']);

        return $this->OrdemServFactory->consultarRelacionamentoComSolicitacaoServico($param);
    }

    public function inserirNovaOrdemServico($parametros) {
        $NRORG = 20;
        $this->connection->beginTransaction();

        try {
            $NRORDEMSERV = $this->newCode->getCodeByTableName("FCM_ORDEMSERV", $NRORG, 1, 10, true);

            $row = $this->FileServerService->uploadImagensManut($parametros['DSCAMINHO']);

            $parametrosOrdem = array(
                "NRORDEMSERV"      => $NRORDEMSERV,
                "NMORDEMOSERV"     => $parametros['NMORDEMOSERV'],
                "NREQUIPAMENTO"    => $parametros['NREQUIPAMENTO'],
                "NRCATEGORIAMANUT" => $parametros['NRCATEGORIAMANUT'],
                "NRTIPOMANUTENCAO" => $parametros['NRTIPOMANUTENCAO'],
                "NRPRIORIDADEMANUT"=> $parametros['NRPRIORIDADEMANUT'],
                "DTORDEMSERV"      => $parametros['DTORDEMSERV'],
                "IDSITUACAO"       => $parametros['IDSITUACAO'],
                "DSORDEMSERV"      => $parametros['DSORDEMSERV'],
                "IDFUNCEXTERNO"    => $parametros['IDFUNCEXTERNO']
            );

            $this->OrdemServFactory->inserirNovaOrdemServico($parametrosOrdem);


            if (count($parametros['IMAGENSSOLICITACAO']) > 0) {
                foreach($parametros['IMAGENSSOLICITACAO'] as $imagens) {
                    if (isset($imagens['NRSIZE'])) {
                        $NRIMGORDEMSERV = $this->newCode->getCodeByTableName('FCM_IMGORDEMSERV', 20, 1, 10, true);

                        $this->OrdemServFactory->inserirImagensNovaOrdem(array(
                            "NRIMGORDEMSERV" => $NRIMGORDEMSERV,
                            "NRORDEMSERV" => $NRORDEMSERV,
                            "DSCAMINHO" => $imagens['src'],
                            "DSTYPE" => $imagens['DSTYPE'],
                            "NRSIZE" => $imagens['NRSIZE']
                        ));
                    }
                }
            }

             // debugApi(['passei']);

            foreach ($row as $file) {
                $NRIMGORDEMSERV = $this->newCode->getCodeByTableName('FCM_IMGORDEMSERV', 20, 1, 10, true);
                $this->OrdemServFactory->inserirImagensNovaOrdem(array(
                    "NRIMGORDEMSERV" => $NRIMGORDEMSERV,
                    "NRORDEMSERV" => $NRORDEMSERV,
                    "DSCAMINHO" => $file['DS'],
                    "DSTYPE" => $file['DSTYPE'],
                    "NRSIZE" => $file['QTSIZE']
                ));
            }

            if ($parametros['SOLICITACAO']) {

                $NRRELSOLICITAORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELSOLICITAORDEMSERV", $NRORG, 1, 10, true);

                $parametrosRelSoli = array(
                    "NRRELSOLICITAORDEMSERV" => $NRRELSOLICITAORDEMSERV,
                    "NRORDEMSERV"             => $NRORDEMSERV,
                    "NRSOLICITACAOSERV"       => $parametros['NRSOLICITACAOSERV']
                );


                $this->OrdemServFactory->inserirRelacionamentoSoliOrdem($parametrosRelSoli);
            }

            forEach($parametros['NRFUNCIONARIOS'] as $Funcionario) {

                $NRRESPONSAVEISORDEMSERV = $this->newCode->getCodeByTableName("FCM_RESPONSAVEISORDEMSERV", $NRORG, 1, 10, true);

                $parametrosFuncionarios = array(
                    "NRRESPONSAVEISORDEMSERV" => $NRRESPONSAVEISORDEMSERV,
                    "NRORDEMSERV"             => $NRORDEMSERV,
                    "NRFUNCIONARIO"           => $Funcionario
                );
                $this->OrdemServFactory->inserirFuncionarioResponsavel($parametrosFuncionarios);
            }

            forEach($parametros['COMENTARIOS'] as $comentario) {

                $NRCOMENTORDEMSERV = $this->newCode->getCodeByTableName("FCM_COMENTORDEMSERV", $NRORG, 1, 10, true);

                $comentarios = array(
                    "NRCOMENTORDEMSERV" => $NRCOMENTORDEMSERV,
                    "NRORDEMSERV"       => $NRORDEMSERV,
                    "NRFUNCCOMENTARIO"  => $comentario['NRFUNCIONARIO'],
                    "DSCOMENTARIO"      => $comentario['DSCOMENTARIO']
                );
                $this->OrdemServFactory->inserirComentarios($comentarios);
            }


            forEach($parametros['NRESCOPOS'] as $Escopo) {

                $NRRELESCORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELESCORDEMSERV", $NRORG, 1, 10, true);

                $escopos = array(
                    "NRRELESCORDEMSERV" => $NRRELESCORDEMSERV,
                    "NRORDEMSERV"       => $NRORDEMSERV,
                    "NRESCOPOMANUT"     => $Escopo
                );

                $this->OrdemServFactory->inserirEscopoRelacionado($escopos);
            }

            $this->connection->commit();
            $response = 'OK';

      } catch(\Exception $e) {
          $this->connection->rollback();
          $response = ($e->getMessage());
      }
       return $response;
    }

    public function insertOrdemServico($parametros) {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $NRORDEMSERV = $this->newCode->getCodeByTableName("FCM_ORDEMSERV", 20, 1, 10, true);

            $parametroOrdem = array(
                "NRORDEMSERV"       => $NRORDEMSERV,
                "NMORDEMOSERV"      => $parametros["NMORDEMOSERV"],
                "NREQUIPAMENTO"     => $parametros["NREQUIPAMENTO"],
                "NRCATEGORIAMANUT"  => $parametros["NRCATEGORIAMANUT"],
                "NRTIPOMANUTENCAO"  => $parametros["NRTIPOMANUTENCAO"],
                "NRPRIORIDADEMANUT" => $parametros["NRPRIORIDADEMANUT"],
                "DTORDEMSERV"       => $parametros["DTORDEMSERV"],
                "IDSITUACAO"        => $parametros["IDSITUACAO"],
                "DSORDEMSERV"       => $parametros["DSORDEMSERV"],
                "IDFUNCEXTERNO"     => $parametros["IDFUNCEXTERNO"],
                "NRORG"             => $nrorg
            );

            $this->OrdemServFactory->insertOrdemServico($parametroOrdem);

            if ($parametros["SOLICITACAO"]) {
                $NRRELSOLICITAORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELSOLICITAORDEMSERV", $nrorg, 1, 10, true);

                $parametroRelacionamento = array(
                    "NRRELSOLICITAORDEMSERV"  => $NRRELSOLICITAORDEMSERV,
                    "NRORDEMSERV"             => $NRORDEMSERV,
                    "NRSOLICITACAOSERV"       => $parametros["NRSOLICITACAOSERV"]
                );

                $this->OrdemServFactory->insertRelSolicitacaoOrdem($parametroRelacionamento);
            }

            foreach ($parametros["NRESCOPOS"] as $NRESCOPOMANUT) {
                $NRRELESCORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELESCORDEMSERV", $nrorg, 1, 10, true);
                
                $parametroEscopo = array(
                    "NRRELESCORDEMSERV" => $NRRELESCORDEMSERV,
                    "NRORDEMSERV"       => $NRORDEMSERV,
                    "NRESCOPOMANUT"     => $NRESCOPOMANUT
                );

                $this->OrdemServFactory->insertRelEscopoOrdem($parametroEscopo);
            }

            if (isset($parametros["FOTOS"])) {
                $parametroFotos = array(
                    "NRORDEMSERV"   => $NRORDEMSERV,
                    "FOTOS"         => $parametros["FOTOS"]
                );
                $this->insertFotosOrdem($parametroFotos);
            }
            
            if (isset($parametros["DSCOMENTARIO"])) {
                $parametroComentario = array(
                    "NRORDEMSERV"       => $NRORDEMSERV,
                    "DSCOMENTARIO"      => $parametros["DSCOMENTARIO"]
                );

                $this->saveComentarioOrdem($parametroComentario);
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            var_dump($ex);
            $this->connection->rollback();
        }
    }

    public function insertFotosOrdem($parametros) {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();

            foreach ($parametros["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_manutencao_fm");

                $NRIMGORDEMSERV = $this->newCode->getCodeByTableName('FCM_IMGORDEMSERV', $nrorg, 1, 10, true);
                $parametroFoto = array(
                    "NRIMGORDEMSERV"    => $NRIMGORDEMSERV,
                    "NRORDEMSERV"       => $parametros["NRORDEMSERV"],
                    "DSCAMINHO"         => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG"             => $nrorg,
                    "DSTYPE"            => $img["type"],
                    "NRSIZE"            => $img["size"]
                );

                $this->OrdemServFactory->insertFotoOrdem($parametroFoto);
            }

            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
        }
    }

    // Função que busca os dados da solicitação de serviço especifica para criar uma nova ordem a partir desta
    public function getSolicitacaoServicoEspecifica($parametros) {

        $param = array(
            'NRSOLICITACAOSERV' => $parametros['NRSOLICITACAOSERV']);

        return $this->OrdemServFactory->getSolicitacaoServicoEspecifica($param);
    }


    public function listOrdemServ($params) {
        $ordemServs = $this->OrdemServFactory->listOrdemServ($params);
        return $ordemServs;
    }

    public function listOrdemServJoin($params) {
        $ordemServs = $this->OrdemServFactory->listOrdemServJoin($params);
        return $ordemServs;
    }
}

?>