/*
@C:\WEB_635\Sql\End_Conv\Oracle\3973_PEDFUNO_VALIDA_EXPEDICAO.SQL
 Tecnisa - Tecnologia em Informtica

 Nome: VALIDA_EXPEDICAO

  Descrio:
  Funo para inconsistncias que impeam a expedicao de um pedido

*/


DROP FUNCTION VALIDA_EXPEDICAO
/

DROP TYPE ERROEXPEDICAO_TABLE
/

DROP TYPE ERROEXPEDICAO_TYPE
/

CREATE TYPE ERROEXPEDICAO_TYPE AS
OBJECT (CDFILIAL VARCHAR2(04),
NRPEDIDO VARCHAR2(10),
NRITEMPED VARCHAR2(10),
DSPRODUTO VARCHAR2(200),
DSERRO VARCHAR2(200));
/

CREATE TYPE ERROEXPEDICAO_TABLE AS
TABLE OF ERROEXPEDICAO_TYPE
/


CREATE OR REPLACE FUNCTION VALIDA_EXPEDICAO
(P_CDFILIAL       IN PEDIDO.CDFILIAL%TYPE,
 P_NRPEDIDO       IN PEDIDO.NRPEDIDO%TYPE,
 P_CDTIPOOPER     IN PEDIDO.CDTIPOOPER%TYPE     DEFAULT NULL,
 P_CDSERIEENTR    IN SERIEENTR.CDSERIEENTR%TYPE DEFAULT NULL,
 P_CDTIPOOPERENTR IN PEDIDO.CDTIPOOPER%TYPE     DEFAULT NULL,
 P_CDEMPRESA      IN INSCRESTAD.CDEMPRESA%TYPE  DEFAULT NULL,
 P_CDINSCESTA     IN INSCRESTAD.CDINSCESTA%TYPE DEFAULT NULL,
 P_DTENTRSAID     IN NOTAFISCAL.DTENTRSAID%TYPE DEFAULT NULL) RETURN ERROEXPEDICAO_TABLE PIPELINED
IS
   V_CDFILIALPED    PEDIDO.CDFILIALPED%TYPE;
   V_IDGERAREPLIC   PAREXPEDICAO.IDGERAREPLIC%TYPE;
   V_CDTIPOOPER     PEDIDO.CDTIPOOPER%TYPE;
   V_CDTIPOOPERUT   PEDIDO.CDTIPOOPER%TYPE;
   V_IDORIGDESTOP   TIPOOPERAC.IDORIGDESTOP%TYPE;
   V_INSCESTADOK    INTEGER;
   V_CDCLIENTE      CLIENTE.CDCLIENTE%TYPE;
   V_IDEMPRESANFEXP PAREXPEDICAO.IDEMPRESANFEXP%TYPE;
   V_CDEMPRESA      INSCRESTAD.CDEMPRESA%TYPE;
   V_CDINSCESTA     INSCRESTAD.CDINSCESTA%TYPE;
   V_IDPEDINDUSTRIA PEDIDO.IDPEDINDUSTRIA%TYPE;
   V_NRORDPROD      ORDPRODUCAO.NRORDPROD%TYPE;
   V_ERRO           VARCHAR2(01);
   V_IDUTILALMOX    PARAMFILIAL.IDUTILALMOX%TYPE;
   V_IDUTILLCESTQ   PARAMFILIAL.IDUTILLCESTQ%TYPE;
   V_IDUTILLOTE     PARAMFILIAL.IDUTILLOTE%TYPE;
   V_CDALMOXARIFE   PARAMFILIAL.CDALMOXARIFE%TYPE;
   V_CDLOCALESTOQ   PARAMFILIAL.CDLOCALESTOQ%TYPE;
   V_DSNUMELOTE     PARAMFILIAL.DSNUMELOTE%TYPE;
   V_DSSEQULOTE     PARAMFILIAL.DSSEQULOTE%TYPE;
   V_CDALMOXPRCLI   PRODFILI.CDALMOXPRCLI%TYPE;
   V_CDLOCALPRCLI   PRODFILI.CDLOCALPRCLI%TYPE;
   V_CDTPOPERPERDA  PAREXPEDICAO.CDTPOPERPERDA%TYPE;
   V_COUNTPERDAPA    INTEGER;
   V_COUNTPERDASE    INTEGER;
   V_IDATUESTTPOP   TIPOOPERAC.IDATUESTTPOP%TYPE;
   OUT_RECORD ERROEXPEDICAO_TYPE := ERROEXPEDICAO_TYPE (NULL, NULL, NULL, NULL, NULL);

BEGIN
   SELECT MAX(CDFILIALPED), MAX(CDTIPOOPER), MAX(CDCLIENTE), NVL(MAX(IDPEDINDUSTRIA), 'N')
     INTO V_CDFILIALPED,    V_CDTIPOOPER,    V_CDCLIENTE,    V_IDPEDINDUSTRIA
     FROM PEDIDO
    WHERE CDFILIAL = P_CDFILIAL
      AND NRPEDIDO = P_NRPEDIDO;

   SELECT NVL(MAX(IDGERAREPLIC), '1'), NVL(MAX(IDEMPRESANFEXP),'EMPRESAFILIAL'),  MAX(CDTPOPERPERDA)
     INTO V_IDGERAREPLIC, V_IDEMPRESANFEXP, V_CDTPOPERPERDA
     FROM PAREXPEDICAO
    WHERE CDFILIAL = P_CDFILIAL;

   V_CDTIPOOPERUT := NVL(P_CDTIPOOPER, V_CDTIPOOPER);

   SELECT NVL(MAX(IDORIGDESTOP), 'C')
     INTO V_IDORIGDESTOP
     FROM TIPOOPERAC
    WHERE (IDENTRSAIDOP = 'S')
      AND (CDTIPOOPER = V_CDTIPOOPERUT);
   IF ((V_CDFILIALPED IS NOT NULL) AND (V_IDGERAREPLIC = '2')) THEN

      ----INICIO SERIEENTR -------
      -- Par. Expedio parametrizado para Gerar Entrada na filial de destino e--
      -- o pedido tem filial de destino e                                      --
      -- Srie da nota de entrada nao foi informada                            --
      IF (P_CDSERIEENTR IS NULL) THEN
      	OUT_RECORD.CDFILIAL  := P_CDFILIAL;
         OUT_RECORD.NRPEDIDO  := P_NRPEDIDO;
         OUT_RECORD.NRITEMPED := NULL;
         OUT_RECORD.DSPRODUTO := NULL;
         OUT_RECORD.DSERRO := 'SERIEENTR';
			PIPE ROW (OUT_RECORD);
		END IF;

      ----FIM SERIEENTR -------

		----INICIO TIPOOPERENTR -------
      -- Par. Expedio parametrizado para Gerar Entrada na filial de destino e--
      -- o pedido tem filial de destino e                                      --
      -- Tp. Operacao de entrada nao foi informada                            --
		IF (P_CDTIPOOPERENTR IS NULL) THEN
			OUT_RECORD.CDFILIAL  := P_CDFILIAL;
			OUT_RECORD.NRPEDIDO  := P_NRPEDIDO;
         OUT_RECORD.NRITEMPED := NULL;
         OUT_RECORD.DSPRODUTO := NULL;
			OUT_RECORD.DSERRO := 'TIPOOPERENTR';
			PIPE ROW (OUT_RECORD);
		END IF;
      ----FIM TIPOOPERENTR -------

      BEGIN
         SELECT IDUTILALMOX,   IDUTILLCESTQ,   IDUTILLOTE,   CDALMOXARIFE,   CDLOCALESTOQ,   DSNUMELOTE,   DSSEQULOTE
           INTO V_IDUTILALMOX, V_IDUTILLCESTQ, V_IDUTILLOTE, V_CDALMOXARIFE, V_CDLOCALESTOQ, V_DSNUMELOTE, V_DSSEQULOTE
           FROM PARAMFILIAL
          WHERE CDFILIAL = V_CDFILIALPED;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_IDUTILALMOX  := 'N';
            V_IDUTILLCESTQ := 'N';
            V_IDUTILLOTE   := 'N';
            V_CDALMOXARIFE := NULL;
            V_CDLOCALESTOQ := NULL;
            V_DSNUMELOTE   := NULL;
            V_DSSEQULOTE   := NULL;
      END;
      ----INICIO ALMOXENTR -------
      IF (V_IDUTILALMOX = 'S') THEN
         FOR C_ITEM IN (SELECT I.NRITEMPED, I.CDPRODUTO, P.CDPRODESTO, P.CDARVPROD, P.NMPRODUTO
                          FROM ITEMPEDIDO I JOIN PRODUTO P
                                              ON I.CDPRODUTO = P.CDPRODUTO
                         WHERE I.CDFILIAL = P_CDFILIAL
                           AND I.NRPEDIDO = P_NRPEDIDO)
         LOOP
            BEGIN
               SELECT CDALMOXPRCLI,   CDLOCALPRCLI
                 INTO V_CDALMOXPRCLI, V_CDLOCALPRCLI
                 FROM PRODFILI PF, PRODUTO PR
                WHERE PR.CDPRODUTO = C_ITEM.CDPRODESTO
                  AND PF.CDFILIAL  = V_CDFILIALPED
                  AND PF.CDPRODUTO = PR.CDPRODESTO;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_CDALMOXPRCLI := NULL;
                  V_CDLOCALPRCLI := NULL;
            END;
            IF (V_CDALMOXPRCLI IS NULL) THEN
               IF (V_CDALMOXARIFE IS NULL) THEN
                  OUT_RECORD.CDFILIAL  := P_CDFILIAL;
                  OUT_RECORD.NRPEDIDO  := P_NRPEDIDO;
                  OUT_RECORD.NRITEMPED := C_ITEM.NRITEMPED;
                  OUT_RECORD.DSPRODUTO := C_ITEM.CDARVPROD||' - '||C_ITEM.NMPRODUTO;
                  OUT_RECORD.DSERRO := 'ALMOXENTR';
                  PIPE ROW (OUT_RECORD);
               END IF;
            END IF;
            ----FIM ALMOXENTR -------

            ----INICIO ALMOXLCESTENTR -------
            IF (V_IDUTILLCESTQ = 'S') THEN
               IF (V_CDLOCALPRCLI IS NULL) THEN
                  IF (V_CDALMOXARIFE <> V_CDALMOXPRCLI) THEN
                     IF NOT(V_CDLOCALESTOQ IS NULL) THEN
                        OUT_RECORD.CDFILIAL  := P_CDFILIAL;
                        OUT_RECORD.NRPEDIDO  := P_NRPEDIDO;
                        OUT_RECORD.NRITEMPED := C_ITEM.NRITEMPED;
                        OUT_RECORD.DSPRODUTO := C_ITEM.CDARVPROD||' - '||C_ITEM.NMPRODUTO;
                        OUT_RECORD.DSERRO := 'ALMOXLCESTENTR';
                        PIPE ROW (OUT_RECORD);
                     END IF;
                  END IF;
               END IF;
            END IF;
            ----FIM ALMOXLCESTENTR -------

            ----INICIO LCESTENTR -------
            IF (V_IDUTILLCESTQ = 'S') THEN
               IF (V_CDLOCALPRCLI IS NULL) THEN
                  IF (V_CDLOCALESTOQ IS NULL) THEN
                     OUT_RECORD.CDFILIAL  := P_CDFILIAL;
                     OUT_RECORD.NRPEDIDO  := P_NRPEDIDO;
                     OUT_RECORD.NRITEMPED := C_ITEM.NRITEMPED;
                     OUT_RECORD.DSPRODUTO := C_ITEM.CDARVPROD||' - '||C_ITEM.NMPRODUTO;
                     OUT_RECORD.DSERRO := 'LCESTENTR';
                     PIPE ROW (OUT_RECORD);
                  END IF;
               END IF;
            END IF;
            ----FIM LCESTENTR -------
         END LOOP;
      END IF;
      ----INICIO LOTEENTR -------
      IF (V_IDUTILLOTE = 'S')  THEN
         IF ((V_DSNUMELOTE IS NULL) OR (V_DSSEQULOTE IS NULL)) THEN
            OUT_RECORD.CDFILIAL  := P_CDFILIAL;
            OUT_RECORD.NRPEDIDO  := P_NRPEDIDO;
            OUT_RECORD.NRITEMPED := NULL;
            OUT_RECORD.DSPRODUTO := NULL;
            OUT_RECORD.DSERRO := 'LOTEENTR';
            PIPE ROW (OUT_RECORD);
         END IF;
      END IF;
      ----FIM LOTEENTR -------

	END IF;
   ----INICIO PEDNFILDEST -------
	--Tp. Operao com destino filial e pedido sem filial de destino informada --
	IF ((V_IDORIGDESTOP = 'L') AND (V_CDFILIALPED IS NULL)) THEN
	   OUT_RECORD.CDFILIAL := P_CDFILIAL;
	   OUT_RECORD.NRPEDIDO := P_NRPEDIDO;
      OUT_RECORD.NRITEMPED := NULL;
      OUT_RECORD.DSPRODUTO := NULL;
	   OUT_RECORD.DSERRO := 'PEDNFILDEST';
	   PIPE ROW (OUT_RECORD);
	END IF;
	----FIM PEDNFILDEST -------

   ----INICIO VINGINSCESTA----
   IF (V_IDEMPRESANFEXP = 'INFORMALANCEXP') THEN
      V_CDEMPRESA  := P_CDEMPRESA;
      V_CDINSCESTA := P_CDINSCESTA;
   ELSIF (V_IDEMPRESANFEXP = 'EMPRESAFILIAL') THEN
      BEGIN
         SELECT CDEMPRESA, CDINSCESTA
           INTO V_CDEMPRESA, V_CDINSCESTA
           FROM FILIAL
          WHERE CDFILIAL = P_CDFILIAL;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_CDEMPRESA  := NULL;
            V_CDINSCESTA := NULL;
      END;
   ELSIF (V_IDEMPRESANFEXP = 'EMPRESACLIENTE') THEN
       BEGIN
         SELECT CDEMPRESA, CDINSCESTA
           INTO V_CDEMPRESA, V_CDINSCESTA
           FROM CLIENTE
          WHERE CDCLIENTE = V_CDCLIENTE;
       EXCEPTION
          WHEN NO_DATA_FOUND THEN
             V_CDEMPRESA  := NULL;
             V_CDINSCESTA := NULL;
       END;
   END IF;
   V_INSCESTADOK := 0;
   SELECT COUNT(1)
      INTO V_INSCESTADOK
      FROM INSCRESTAD I
     WHERE (I.CDEMPRESA  = V_CDEMPRESA)
       AND (I.CDINSCESTA = V_CDINSCESTA)
       AND (NVL(P_DTENTRSAID, TRUNC(SYSDATE)) >= NVL(NVL(I.DTINIVIGIEST, P_DTENTRSAID), TRUNC(SYSDATE)))
       AND (NVL(P_DTENTRSAID, TRUNC(SYSDATE)) <= NVL(NVL(I.DTFIMVIGIEST, P_DTENTRSAID), TRUNC(SYSDATE)));
   IF (V_INSCESTADOK = 0) THEN
      OUT_RECORD.CDFILIAL := P_CDFILIAL;
      OUT_RECORD.NRPEDIDO := P_NRPEDIDO;
      OUT_RECORD.NRITEMPED := NULL;
      OUT_RECORD.DSPRODUTO := NULL;
      OUT_RECORD.DSERRO := 'VINGINSCESTA';
      PIPE ROW (OUT_RECORD);

   END IF;

   ----FIM VINGINSCESTA----

   ----INICIO PEDIDO DE INDUSTRIALIZACAO----
   IF (V_IDPEDINDUSTRIA = 'S') THEN
      ----INICIO ORDEM DE PRODUO DO PEDIDO DE INDUSTRIALIZACAO NO EXISTE OU NAO FOI CONCLUIDA----
      V_ERRO := '0';
      BEGIN
         SELECT MAX(O.NRORDPROD)
           INTO V_NRORDPROD
           FROM ITEMPEDIDO I JOIN ORDPRODUCAO O
                               ON I.CDFILIAL = O.CDFILIAL
                              AND I.NRPEDIDO = O.NRPEDIDO
                              AND I.NRITEMPED = O.NRITEMPED
          WHERE I.CDFILIAL = P_CDFILIAL
            AND I.NRPEDIDO = P_NRPEDIDO
            AND I.IDITEMPEDTERC = 'S';
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_NRORDPROD := NULL;
      END;
      IF (V_NRORDPROD IS NULL) THEN
         V_ERRO := '1';
      ELSE
         FOR C_OP IN (SELECT O.CDFILIAL, O.NRORDPROD, O.NRSUBORDPROD, O.IDSITORDPROD
                        FROM ITEMPEDIDO I JOIN ORDPRODUCAO O
                                            ON I.CDFILIAL = O.CDFILIAL
                                           AND I.NRPEDIDO = O.NRPEDIDO
                                           AND I.NRITEMPED = O.NRITEMPED
                       WHERE I.CDFILIAL = P_CDFILIAL
                         AND I.NRPEDIDO = P_NRPEDIDO
                         AND I.IDITEMPEDTERC = 'S')
         LOOP
            IF C_OP.IDSITORDPROD <> '3' THEN
              V_ERRO := '1';
              EXIT;
            END IF;
         END LOOP;
      END IF;
      IF (V_ERRO = '1') THEN
         OUT_RECORD.CDFILIAL := P_CDFILIAL;
         OUT_RECORD.NRPEDIDO := P_NRPEDIDO;
         OUT_RECORD.NRITEMPED := NULL;
         OUT_RECORD.DSPRODUTO := NULL;
         OUT_RECORD.DSERRO := 'PEDINDSEMOP';
         PIPE ROW (OUT_RECORD);

      END IF;
      ----FIM ORDEM DE PRODUO DO PEDIDO DE INDUSTRIALIZACAO NO EXISTE OU NAO FOI CONCLUIDA----

      ----INICIO ORDEM DE PRODUO DO PEDIDO DE INDUSTRIALIZACAO CONCLUIDA COM QT MENOR QUE A QT PEDIDO----
      FOR C_QTOP IN (SELECT I.CDFILIAL, I.NRPEDIDO, I.NRITEMPED, I.QTITEMPED, O.NRORDPROD, O.NRSUBORDPROD, O.QTAPRODPREOP, P.QTPRACABADO
                       FROM ITEMPEDIDO I JOIN ORDPRODUCAO O
                                           ON I.CDFILIAL = O.CDFILIAL
                                          AND I.NRPEDIDO = O.NRPEDIDO
                                          AND I.NRITEMPED = O.NRITEMPED
                                         JOIN PRODACABADO P
                                           ON O.CDFILIAL = P.CDFILIAL
                                          AND O.NRORDPROD = P.NRORDPROD
                                          AND O.NRSUBORDPROD = P.NRSUBORDPROD
                                          AND O.CDPRODUTO = P.CDSUBPRODETA
                      WHERE I.CDFILIAL = P_CDFILIAL
                        AND I.NRPEDIDO = P_NRPEDIDO
                        AND I.CDPRODUTO = O.CDPRODUTO
                        AND O.IDSITORDPROD = '3'
                        AND I.IDITEMPEDTERC = 'S'
                        AND ROUND(P.QTPRACABADO,5) < ROUND(I.QTITEMPED, 5))
      LOOP
         OUT_RECORD.CDFILIAL := P_CDFILIAL;
         OUT_RECORD.NRPEDIDO := P_NRPEDIDO;
         OUT_RECORD.NRITEMPED := C_QTOP.NRITEMPED;
         OUT_RECORD.DSPRODUTO := NULL;
         OUT_RECORD.DSERRO := 'QTOPMENORPED';
         PIPE ROW (OUT_RECORD);
      END LOOP;
      ----FIM ORDEM DE PRODUO DO PEDIDO DE INDUSTRIALIZACAO CONCLUIDA COM QT MENOR QUE A QT PEDIDO----

      ----INICIO ORDEM DE PRODUO DO PEDIDO DE INDUSTRIALIZACAO POSSUI PERDA E NO TEM TIPO DE OPERACAO DE PERDA PARAMETRIZADO----
      V_COUNTPERDAPA := 0;
      V_COUNTPERDASE := 0;
      BEGIN
         SELECT COUNT(1)
           INTO V_COUNTPERDAPA
           FROM TRABPRODACABPED T JOIN PERDAOP P
                                    ON T.CDFILIAL = P.CDFILIAL
                                   AND T.NRORDPROD = P.NRORDPROD
                                   AND T.NRSUBORDPROD = P.NRSUBORDPROD
          WHERE T.CDFILIAL = P_CDFILIAL
            AND T.NRPEDIDO = P_NRPEDIDO;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_COUNTPERDAPA := 0;
      END;

      BEGIN
         SELECT COUNT(1)
           INTO V_COUNTPERDASE
           FROM TRABSEMIACABADOPED T JOIN PERDAOP P
                                       ON T.CDFILIAL = P.CDFILIAL
                                      AND T.NRORDPROD = P.NRORDPROD
                                      AND T.NRSUBORDPROD = P.NRSUBORDPROD
          WHERE T.CDFILIAL = P_CDFILIAL
            AND T.NRPEDIDO = P_NRPEDIDO;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_COUNTPERDASE := 0;
      END;

      IF ((V_COUNTPERDAPA > 0) OR (V_COUNTPERDASE > 0)) THEN
         IF (V_CDTPOPERPERDA IS NULL) THEN
            OUT_RECORD.CDFILIAL := P_CDFILIAL;
            OUT_RECORD.NRPEDIDO := P_NRPEDIDO;
            OUT_RECORD.NRITEMPED := NULL;
            OUT_RECORD.DSPRODUTO := NULL;
            OUT_RECORD.DSERRO := 'TPOPPERDA';
            PIPE ROW (OUT_RECORD);
      ----FIM ORDEM DE PRODUO DO PEDIDO DE INDUSTRIALIZACAO POSSUI PERDA E NO TEM TIPO DE OPERACAO DE PERDA PARAMETRIZADO----
         ELSE
      ----INICIO ORDEM DE PRODUO DO PEDIDO DE INDUSTRIALIZACAO POSSUI PERDA E TIPO DE OPERACAO DE PERDA ATUALIZA ESTOQUE----
            SELECT NVL(MAX(IDATUESTTPOP), 'S')
              INTO V_IDATUESTTPOP
              FROM TIPOOPERAC
             WHERE IDENTRSAIDOP = 'S'
               AND CDTIPOOPER = V_CDTPOPERPERDA;
            IF (V_IDATUESTTPOP = 'S') THEN
               OUT_RECORD.CDFILIAL := P_CDFILIAL;
               OUT_RECORD.NRPEDIDO := P_NRPEDIDO;
               OUT_RECORD.NRITEMPED := NULL;
               OUT_RECORD.DSPRODUTO := NULL;
               OUT_RECORD.DSERRO := 'TPPERDAEST';
               PIPE ROW (OUT_RECORD);
            END IF;
         END IF;
      END IF;
      ----FIM ORDEM DE PRODUO DO PEDIDO DE INDUSTRIALIZACAO POSSUI PERDA E TIPO DE OPERACAO DE PERDA ATUALIZA ESTOQUE----
   END IF;
   ----FIM PEDIDO DE INDUSTRIALIZACAO----
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('VALIDA_EXPEDICAO','6.49.028'); END;
/
