/* eslint-disable*/
import {
  Metadata, Http, IDictionary, Router,
} from '@zeedhi/core';
import {
  AlertService, Button, Card, Col, Checkbox, Date as ZdDate, DialogService, FileInput, Form, Select, SelectMultiple, Text,
  Textarea, TextInput,
} from '@zeedhi/common';
import {
  IterableCarousel, IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import {
  FileUpload,
} from './FileUpload';

export class ControllerOrdemServico {
  private dadosOriginal: any = {
    'iterable-cards-ordemServico': [],
  };

  private dadosComentarios: any = {};

  private imagensSolicitacao: any = [];

  public proximaPagina({ component }: any) {
    component.parent.datasource.page += 1;
    if (component.parent.paginationLength === component.parent.datasource.page) {
      component.disabled = true;
    }
    component.parent.datasource.get();
    Metadata.getInstance<Button>(`pagina-anterior-botao_${component.name.split('_')[1]}`).disabled = false;
  }

  public paginaAnterior({ component }: any) {
    if (component.parent.datasource.page > 1) {
      component.parent.datasource.page -= 1;
      if (component.parent.datasource.page === 1) {
        component.disabled = true;
      }
      component.parent.datasource.get();
      Metadata.getInstance<Button>(`proxima-pagina-botao_${component.name.split('_')[1]}`).disabled = false;
    }
  }

  // Função que desabilita os botões quando tem apenas uma página nos cards
  public maisQueUmaPagina({ component }: any) {
    component.disabled = !(Metadata.getInstance<IterableComponentRender>(component.name.split('_')[1]).paginationLength > 1);
  }

  public filtrarDados(dados: IDictionary, valoresASerFiltrado: any, colunaAFiltrar: any) {
    const dadosFiltrados: any = [];
    dados.forEach((dadoUnitario: any) => {
      if (dadoUnitario[colunaAFiltrar] !== null) {
        String(dadoUnitario[colunaAFiltrar]).split(',').forEach((dadosColuna: any) => {
          if (valoresASerFiltrado.includes(dadosColuna)) {
            dadosFiltrados.push(dadoUnitario);
            return false;
          }
        });
      }
    });
    return dadosFiltrados;
  }

  public async filtrar({ component }: any) {
    const nomes = component.name.split('_');
    const iterable = Metadata.getInstance<IterableComponentRender>(nomes[1]);
    iterable.datasource.updateData(this.dadosOriginal[nomes[1]]);
    iterable.datasource.searchIn.forEach((colunas: string) => {
      const select = Metadata.getInstance<SelectMultiple>(`CampoFiltro_${nomes[1]}_${colunas}`).value;
      if (select.length) {
        const dadosIterable = (iterable.datasource as any);
        iterable.datasource.updateData(this.filtrarDados(dadosIterable.allData, select, colunas));
      }
    });
    this.maisQueUmaPagina({
      component: Metadata.getInstance<Button>(`proxima-pagina-botao_${nomes[1]}`),
    });
  }

  public messages(returnMessage: string, mensagem: string, textoErro: string) {
    if (returnMessage[0] === 'OK') {
      AlertService.show({
        name: 'basic-alert',
        text: `${mensagem}`,
        top: true,
        left: false,
        timeout: 3000,
        color: 'success',
      });
    } else {
      DialogService.show({
        name: 'warning-dialog',
        type: 'warning',
        title: 'Erro!',
        text: `Operação bloqueada! ${textoErro} <br><br>Se o problema persistir entre em contato com o suporte.`,
      });
    }
  }

  public alertaPreencherCampos() {
    AlertService.show({
      name: 'basic-alert',
      text: 'Preencha todos os campos obrigatórios (*)!',
      top: true,
      left: false,
      timeout: 3000,
      color: 'error',
    });
  }

  // -------- FUNÇÕES DA TELA DE NOVA ORDEM DE SERVIÇO
  public desabilitarTecnicos({ component }:{ component: Checkbox }) {
    const checkbox = component.value;
    const selectTecnicosNovaOrdem = Metadata.getInstance<SelectMultiple>('sel-tecnico-novaOrdemServico');

    if (checkbox) {
      selectTecnicosNovaOrdem.isVisible = false;
      selectTecnicosNovaOrdem.removeValidation('required');
    } else {
      selectTecnicosNovaOrdem.isVisible = true;
      selectTecnicosNovaOrdem.addValidation('required');
    }
  }

  public acionaCarroselImagens({ component }:{ component: Button }) {
    Metadata.getInstance<Card>('card-imagens').isVisible = true;
    component.isVisible = false;
    Metadata.getInstance<Form>('form-descricao').children[0]!.grid!.cols = '12';
  }

  // Funções que populam os filtros tanto na tela de nova ordem de serviço, quanto na tela de ordem de serviço
  public getEquipamentos({ component }:{ component: Select }) {
    Http.get('/Equipamentos').then((response: IDictionary) => {
      component.datasource.data = response.data.dataset.equipamentos;
    });
  }

  public getCategorias({ component }:{ component: Select }) {
    Http.get('/CategoriaManut').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.categorias[0].map((elemento: any) => ({ value: elemento.NRCATEGORIAMANUT, name: elemento.NMCATEGORIAMANUT })));
    });
  }

  public getTiposManutencao({ component }:{ component: Select }) {
    Http.get('/TipoManutencao').then((response: IDictionary) => {
      response.data.dataset.tipoManut[0].forEach((dados :IDictionary) => {
        component.datasource.data.push({
          value: dados.NRTIPOMANUTENCAO,
          name: dados.NMTIPOMANUTENCAO,
        });
      });
    });
  }

  public getPrioridades({ component }:{ component: Select }) {
    Http.get('/PrioridadeManut').then((response: IDictionary) => {
      response.data.dataset.prioridade[0].forEach((dados :IDictionary) => {
        component.datasource.data.push({
          value: dados.NRPRIORIDADEMANUT,
          name: dados.NMPRIORIDADEMANUT,
        });
      });
    });
  }

  public getEscopos({ component }:{ component: Select }) {
    Http.get('/EscopoManut').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.escopo[0].map((elemento: any) => ({ value: elemento.NRESCOPOMANUT, name: elemento.NMESCOPOMANUT })));
    });
  }

  public getFuncionarios({ component }:{ component: SelectMultiple }) {
    Http.get('/Funcionarios').then((response: IDictionary) => {
      component.datasource.data = response.data.dataset.employee;
    });
  }

  public setDataAtual() {
    const date = new Date();
    Metadata.getInstance<ZdDate>('input-data').value = date.toLocaleDateString();
  }

  public async novaOrdemServico() {
    const form = Metadata.getInstance<Form>('form-selects').validate();
    const formDescricao = Metadata.getInstance<Form>('form-descricao').validate();
    const checkbox = Metadata.getInstance<Checkbox>('checkbox-profissionalExterno-novaOrdem').value;

    if (form && formDescricao) {
      const imagens = await this.enviarImagens('FileInput-ImagensNovaOrdem_carousel-imagens-novaOrdem');
      Http.post('/NovaOrdemServico', {
        NMORDEMOSERV: Metadata.getInstance<TextInput>('nome-ordemservico').value,
        NREQUIPAMENTO: Metadata.getInstance<Select>('select-equipamento').value,
        NRCATEGORIAMANUT: Metadata.getInstance<Select>('select-categoria').value,
        NRTIPOMANUTENCAO: Metadata.getInstance<Select>('select-tipo').value,
        NRPRIORIDADEMANUT: Metadata.getInstance<Select>('select-prioridade').value,
        IDSITUACAO: Metadata.getInstance<Select>('select-status').value,
        DTORDEMSERV: Metadata.getInstance<ZdDate>('input-data').value,
        IDFUNCEXTERNO:(checkbox) ? 'S' : 'N',
        DSORDEMSERV: Metadata.getInstance<Textarea>('textarea-descricaoNovaOrdem').value,
        NRFUNCIONARIOS: Metadata.getInstance<SelectMultiple>('sel-tecnico-novaOrdemServico').value,
        NRESCOPOS: Metadata.getInstance<SelectMultiple>('select-escopo').value,
        DSCAMINHO: imagens,
        COMENTARIOS: this.dadosComentarios,
        SOLICITACAO: false,
        IMAGENSSOLICITACAO: this.imagensSolicitacao,
      }).then((response: IDictionary) => {
        const mensagemRetorno = response.data.dataset.novaOrdem;
        const mensagem = 'Nova ordem de serviço adicionada com sucesso!';
        const textoErro = 'Esta ordem de serviço não pôde ser adicionada.';
        this.messages(mensagemRetorno, mensagem, textoErro);
      }).then(() => {
        setTimeout(() => {
          Router.replace('/ordemdeservico');
        }, 2000);
      });
    } else {
      this.alertaPreencherCampos();
    }
  }

  public exibirImagens({ component }: any) {
    const carousel = Metadata.getInstance<IterableCarousel>(component.name.split('_')[1]);

    const imagensAExibir: any = this.imagensSolicitacao;
    if (component.value.length) { // tamanho do fileInput
      component.value.forEach((imagem: any, pos: number) => {
        imagensAExibir.push({
          id: pos,
          src: window.URL.createObjectURL(imagem), // cria uma url local para a imagem
        });
      });
      carousel.isVisible = true;
    } else {
      carousel.isVisible = false;
    }
    carousel.iterable.datasource.updateData(imagensAExibir);
  }

  public async enviarImagens(name: string) {
    const fileInputObject = Metadata.getInstance<FileInput>(name).value;
    let arquivos: any = [];
    if (fileInputObject && fileInputObject.length > 0) {
      const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
      const buildFilesPromises = fileInputObject.map((file: any) => {
        if (types.includes(file.type)) {
          return FileUpload.buildFileSendObject(file);
        }
      });
      await Promise.all(buildFilesPromises).then((files: any) => {
          arquivos  = files;
      });
    }
    return arquivos;
  }

  // ----------- FUNÇÕES DA TELA ORDENS DE SERVIÇO
  // Cria os cards de todas as ordens de serviço
  public criarCardsOrdensServico() {
    // const textPrioridade = Metadata.getInstance<Text>('text-prioridade');

    Http.get('/OrdensServico').then((response: IDictionary) => {
      const dadosOrdemServico = response.data.dataset.ordensServico[0];
      const codigo = dadosOrdemServico.map((dados: any) => String(dados.NRORDEMSERV));
      const prioridade = dadosOrdemServico.map((dados: any) => String(dados.NMPRIORIDADEMANUT));
      const nomeFuncionario = (dadosOrdemServico.map((dados: any) => (dados.NMFUNCIONARIO !== null) ? dados.NMFUNCIONARIO.split(',') : ' ' )).flat(1);
      const status = dadosOrdemServico.map((dados: any) => String(dados.NMIDSITUACAO));
      const data = dadosOrdemServico.map((dados: any) => String(dados.DTORDEMSERV));
      const tipoManutencao = dadosOrdemServico.map((dados: any) => String(dados.NMTIPOMANUTENCAO));
      const categoria = dadosOrdemServico.map((dados: any) => String(dados.NMCATEGORIAMANUT));
      const equipamento = dadosOrdemServico.map((dados: any) => String(dados.NMEQUIPAMENTO));

      Metadata.getInstance<SelectMultiple>('CampoFiltro_iterable-cards-ordemServico_NRORDEMSERV').updateItems(codigo.filter((nome: string, pos: number) => codigo.indexOf(nome) === pos));
      Metadata.getInstance<SelectMultiple>('CampoFiltro_iterable-cards-ordemServico_NMPRIORIDADEMANUT').updateItems(prioridade.filter((nome: string, pos: number) => prioridade.indexOf(nome) === pos));
      Metadata.getInstance<SelectMultiple>('CampoFiltro_iterable-cards-ordemServico_NMFUNCIONARIO').updateItems(nomeFuncionario.filter((nome: string, pos: number) => nomeFuncionario.indexOf(nome) === pos));
      Metadata.getInstance<SelectMultiple>('CampoFiltro_iterable-cards-ordemServico_NMIDSITUACAO').updateItems(status.filter((nome: string, pos: number) => status.indexOf(nome) === pos));
      Metadata.getInstance<SelectMultiple>('CampoFiltro_iterable-cards-ordemServico_DTORDEMSERV').updateItems(data.filter((nome: string, pos: number) => data.indexOf(nome) === pos));
      Metadata.getInstance<SelectMultiple>('CampoFiltro_iterable-cards-ordemServico_NMTIPOMANUTENCAO').updateItems(tipoManutencao.filter((nome: string, pos: number) => tipoManutencao.indexOf(nome) === pos));
      Metadata.getInstance<SelectMultiple>('CampoFiltro_iterable-cards-ordemServico_NMCATEGORIAMANUT').updateItems(categoria.filter((nome: string, pos: number) => categoria.indexOf(nome) === pos));
      Metadata.getInstance<SelectMultiple>('CampoFiltro_iterable-cards-ordemServico_NMEQUIPAMENTO').updateItems(equipamento.filter((nome: string, pos: number) => equipamento.indexOf(nome) === pos));

      // textPrioridade.text  = `<span color: ${dados[0].DSCORPRIORIDADE}</span>`;
      this.dadosOriginal['iterable-cards-ordemServico'] = dadosOrdemServico;
      Metadata.getInstance<IterableComponentRender>('iterable-cards-ordemServico').datasource.updateData(dadosOrdemServico);
      this.maisQueUmaPagina({
        component: Metadata.getInstance<Button>('proxima-pagina-botao_iterable-cards-ordemServico'),
      });
    });
  }

  public imprimirOrdemDeServico({ component }:{ component: Button }) {
    Http.post('/OrdemServicoEspecifica', { NRORDEMSERV: component.name.split('_')[1] }).then((response: IDictionary) => {
      const dadosOS = response.data.dataset.ordem;

      const newWindow = window.open();
      const ordemHTML = `
      <div>
        <center>
          <h3> Ordem de Serviço </h3>
        </center>
        <div style="display: flex;">
          <div style="margin-right: 4%">
              <h5>
              Cód da Ordem
              </h5>
            <p> ${dadosOS[0].NRORDEMSERV} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Categoria
            </h5>
            <p> ${dadosOS[0].NMCATEGORIAMANUT} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Tipo de Manutenção
            </h5>
            <p> ${dadosOS[0].NMTIPOMANUTENCAO} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Prioridade
            </h5>
            <p> ${dadosOS[0].NMPRIORIDADEMANUT} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Status
            </h5>
            <p> ${dadosOS[0].NMIDSITUACAO} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Data
            </h5>
            <p> ${dadosOS[0].DTORDEMSERV} </p>
          </div>
        </div>

        <div>
          <h5>
            Equipamento
          </h5>
          <p> ${dadosOS[0].NMEQUIPAMENTO} </p>
        </div>

        <div>
          <h5>
            Responsáveis
          </h5>
          <p> ${(dadosOS[0].NMFUNCIONARIO === ' ') ? 'Executado por um profissional externo' : dadosOS[0].NMFUNCIONARIO} </p>
        </div>

        <div>
          <h5>
            Descrição
          </h5>
          <p> ${dadosOS[0].DSORDEMSERV} </p>
        </div>

      </div>
    `;
      newWindow?.document.write(ordemHTML);
      newWindow?.print();
    });
  }

  // monta as informações da tela da visualizacao da ordem de servico
  public montarInformacoesOrdemServico() {
    const NRORDEMSERV = window.location.search.slice(1);
    console.log(NRORDEMSERV);

    Http.post('/OrdemServicoEspecifica', { NRORDEMSERV }).then((response: IDictionary) => {
      const dados = response.data.dataset.ordem;
      const carousel = Metadata.getInstance<IterableCarousel>('carousel-imagens-visualizar');
      const imagensAExibir: any = [];

      Metadata.getInstance<Text>('conteudos-informacao-equipamento').text = dados[0].NMEQUIPAMENTO;
      Metadata.getInstance<Text>('conteudos-informacao-categoriaManut').text = dados[0].NMCATEGORIAMANUT;
      Metadata.getInstance<Text>('conteudo-informacao-tipoManut').text = dados[0].NMTIPOMANUTENCAO;
      Metadata.getInstance<Text>('conteudo-informacao-prioridade').text = dados[0].NMPRIORIDADEMANUT;
      Metadata.getInstance<Text>('conteudos-informacao-status').text = dados[0].NMIDSITUACAO;
      Metadata.getInstance<Text>('conteudos-informacao-dataOrdem').text = dados[0].DTORDEMSERV;
      Metadata.getInstance<Text>('conteudos-informacao-tecnicos').text = (dados[0].NMFUNCIONARIO !== ' ') ? dados[0].NMFUNCIONARIO : 'Executado por um profissional externo';
      Metadata.getInstance<Text>('cod-da-pagina-visualizacaoOrdem').text = `#${dados[0].NRORDEMSERV}`;
      Metadata.getInstance<Text>('conteudos-informacao-escopo').text = dados[0].NMESCOPOMANUT;

      Metadata.getInstance<Textarea>('textarea-visualizacaoOrdem').value = dados[0].DSORDEMSERV;

      dados.forEach((elemento: any) => {
        if (elemento.DSCAMINHO !== null) {
          imagensAExibir.push({
            id: elemento.NRIMGORDEMSERV,
            src: elemento.DSCAMINHO,
          });
        }
      });

      if (imagensAExibir.length) {
        Metadata.getInstance<Card>('card-imagens-visualizacaoOrdem').isVisible = true;
      }
      carousel.iterable.datasource.updateData(imagensAExibir);
    });
  }

  // cria os cards do comentário da visualizacao da ordem de serviço
  public async criarCardsComentariosOrdem() {
    const NRORDEMSERV = window.location.search.slice(1);

    Http.post('/ComentariosOrdem', { NRORDEMSERV }).then((response: IDictionary) => {
      const dadosComentarios = response.data.dataset.comentarios[0];

      Metadata.getInstance<IterableComponentRender>('iterable-cards-visualizacaoOrdem').datasource.updateData(dadosComentarios);
      this.maisQueUmaPagina({
        component: Metadata.getInstance<Button>('proxima-pagina-botao_iterable-cards-visualizacaoOrdem'),
      });
    });
  }

  // Função que exibe um card com as informações da solicitação de serviço caso a ordem esteja vinculada a uma
  public exibirRelacionamentoComSolicitacao() {
    const NRORDEMSERV = window.location.search.slice(1);
    const cardSolicitacao =  Metadata.getInstance<Card>('cards-informacoes-solicitacao');

    Http.post('/RelacionamentoSScomOS', { NRORDEMSERV }).then((response: IDictionary) => {
      const dadosSolicitacao = response.data.dataset.solicitacao[0];

      if (dadosSolicitacao.length) {
        cardSolicitacao.isVisible = true;
        Metadata.getInstance<Text>('text-data-solicitacaoServico').text = dadosSolicitacao[0].DTSOLICITACAOSERV;
        Metadata.getInstance<Text>('text-solicitante-solicitacaoServico').text = dadosSolicitacao[0].NMFUNCIONARIO;
        Metadata.getInstance<Text>('text-impactos-solicitacaoServico').text = dadosSolicitacao[0].NMIMPACTOMANUT;
        Metadata.getInstance<Text>('text-solicitacaoServico').text = `#${dadosSolicitacao[0].NRSOLICITACAOSERV}`;
      } else {
        cardSolicitacao.isVisible = false;
      }
    });
  }

  // Função para adicionar os comentários na tela de visualização da ordem de serviço
  public adicionarComentarios() {
    const comentario = Metadata.getInstance<Textarea>('textarea-adicionar-comentario');

    if(comentario.value.replaceAll(' ', '') !== '') {
      Http.post('/AdicionarComentarios', {
        NRORDEMSERV: window.location.search.slice(1),
        NRFUNCCOMENTARIO: 1,
        DSCOMENTARIO: comentario.value,
      }).then((response: IDictionary) => {
        const mensagemRetorno = response.data.dataset.comentarios;
        const mensagem = 'Comentário adicionado com sucesso!';
        const textoErro = 'Erro ao adicionar comentário.';
        this.messages(mensagemRetorno, mensagem, textoErro);
      }).then(() => {
        this.criarCardsComentariosOrdem();
        comentario.value = '';
      });
    } else {
      AlertService.show({
        name: 'basic-alert',
        text: `Escreva algum comentário válido`,
        top: true,
        left: false,
        timeout: 3000,
        color: 'warning',
      });
    }
  }

  public adicionarComentarioNovaOrdem() {
    const DSCOMENTARIO = Metadata.getInstance<Textarea>('adicionar-comentario-novaOrdem');
    if (DSCOMENTARIO.value.replaceAll(' ', '') !== '') {
      const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios-novaOrdemServico');
      let dadosIterable = (iterable.datasource as any);
      dadosIterable = (dadosIterable.allData || dadosIterable.data);
      dadosIterable.unshift({
        NMFUNCIONARIO: 'Maria Eduarda Machado',
        NRFUNCIONARIO: '1',
        DTCOMENTARIO: (new Date()).toLocaleDateString(),
        DSCOMENTARIO: DSCOMENTARIO.value,
      });
      iterable.datasource.updateData(dadosIterable);
      this.dadosComentarios = dadosIterable;
      DSCOMENTARIO.value = '';
    }
  }

  public montarInformacoesSolicitacaoServico() {
    const NRSOLICITACAOSERV = window.location.search.slice(1);

    Http.post('/getSolicitacaoServicoEspecifica', { NRSOLICITACAOSERV }).then((response: IDictionary) => {
      const dados = response.data.dataset.solicitacao;
      const carousel = Metadata.getInstance<IterableCarousel>('carousel-imagens-converterEmOrdem');
      const imagensAExibir: any = [];

      // Cabeçalho da solicitação
      Metadata.getInstance<Text>('text-solicitacaoServico').text = `#${dados[0].NRSOLICITACAOSERV}`;
      Metadata.getInstance<Text>('text-data-solicitacaoServico').text = dados[0].DTSOLICITACAOSERV;
      Metadata.getInstance<Text>('text-solicitante-solicitacaoServico').text = dados[0].NMFUNCIONARIO;
      Metadata.getInstance<Text>('text-impactos-solicitacaoServico').text = dados[0].NMIMPACTOMANUT;

      Metadata.getInstance<Select>('select-equipamento-converterEmOrdem').value = dados[0].NREQUIPAMENTO;
      Metadata.getInstance<Select>('select-categoria-converterEmOrdem').value = dados[0].NRCATEGORIAMANUT;
      Metadata.getInstance<Select>('select-prioridade-converterEmOrdem').value = dados[0].NRPRIORIDADEMANUT;
      Metadata.getInstance<Textarea>('textarea-descricaoConversaoEmOrdem').value = dados[0].DSSOLICITACAOSERV;

      imagensAExibir.push({
        id: dados[0].NRIMGSOLICITACAOSERV,
        src: dados[0].DSCAMINHO,
        DSTYPE: dados[0].DSTYPE,
        NRSIZE: dados[0].NRSIZE,
      });
      this.imagensSolicitacao = imagensAExibir;
      carousel.iterable.datasource.updateData(imagensAExibir);
    });
  }

  public desabilitarTecnicosConverterEmOrdem({ component }:{ component: Checkbox }) {
    const checkbox = component.value;
    const selectTecnicosNovaOrdem = Metadata.getInstance<SelectMultiple>('sel-tecnico-converterEmOrdem');

    if (checkbox) {
      selectTecnicosNovaOrdem.isVisible = false;
      selectTecnicosNovaOrdem.removeValidation('required');
    } else {
      selectTecnicosNovaOrdem.isVisible = true;
      selectTecnicosNovaOrdem.addValidation('required');
    }
  }

  public async novaOrdemServicoConvertida() {
    const form = Metadata.getInstance<Form>('form-selects-converterEmOrdem').validate();
    const formDescricao = Metadata.getInstance<Form>('form-descricao-converterEmOrdem').validate();
    const checkbox = Metadata.getInstance<Checkbox>('checkbox-profissionalExterno-converterEmOrdem').value;

    if (form && formDescricao) {
      const imagens = await this.enviarImagens('FileInput-ImagensNovaOrdem_carousel-imagens-converterEmOrdem');
      Http.post('/NovaOrdemServicoConvertida', {
        NRSOLICITACAOSERV: window.location.search.slice(1),
        NMORDEMOSERV: Metadata.getInstance<TextInput>('nome-ordemservico-converterEmOrdem').value,
        NREQUIPAMENTO: Metadata.getInstance<Select>('select-equipamento-converterEmOrdem').value,
        NRCATEGORIAMANUT: Metadata.getInstance<Select>('select-categoria-converterEmOrdem').value,
        NRTIPOMANUTENCAO: Metadata.getInstance<Select>('select-tipo-converterEmOrdem').value,
        NRPRIORIDADEMANUT: Metadata.getInstance<Select>('select-prioridade-converterEmOrdem').value,
        IDSITUACAO: Metadata.getInstance<Select>('select-status-converterEmOrdem').value,
        DTORDEMSERV: Metadata.getInstance<ZdDate>('input-data').value,
        IDFUNCEXTERNO:(checkbox) ? 'S' : 'N',
        DSORDEMSERV: Metadata.getInstance<Textarea>('textarea-descricaoConversaoEmOrdem').value,
        NRFUNCIONARIOS: Metadata.getInstance<SelectMultiple>('sel-tecnico-converterEmOrdem').value,
        NRESCOPOS: Metadata.getInstance<SelectMultiple>('select-escopo-converterEmOrdem').value,
        DSCAMINHO: imagens,
        IMAGENSSOLICITACAO: this.imagensSolicitacao,
        COMENTARIOS: this.dadosComentarios,
        SOLICITACAO: true,
      }).then((response: IDictionary) => {
        const mensagemRetorno = response.data.dataset.novaOrdem;
        const mensagem = 'Nova ordem de serviço adicionada com sucesso!';
        const textoErro = 'Esta ordem de serviço não pôde ser adicionada.';
        this.messages(mensagemRetorno, mensagem, textoErro);
      }).then(() => {
        setTimeout(() => {
          Router.replace('/ordemdeservico');
        }, 2000);
      });
    } else {
      this.alertaPreencherCampos();
    }
  }
}
