import { Button } from '@zeedhi/common';
import {
  Http, IDictionary, Router, Metadata,
} from '@zeedhi/core';
import {
  IterableComponentRender,
} from '@zeedhi/teknisa-components-common';

export class DashboardController {
  public redirecionarCriarOS({ component }:{ component: Button }) {
    Router.replace(`/solicitacao?${component.name.split('_')[1]}`);
  }

  public imprimirOrdemDeServico({ component }:{ component: Button }) {
    Http.post('/OrdemServicoEspecifica', { NRORDEMSERV: component.name.split('_')[1] }).then((response: IDictionary) => {
      const dadosOS = response.data.dataset.ordem;

      const newWindow = window.open();
      const ordemHTML = `
      <div>
        <center>
          <h3> Ordem de Serviço </h3>
        </center>
        <div style="display: flex;">
          <div style="margin-right: 4%">
              <h5>
              Cód da Ordem
              </h5>
            <p> ${dadosOS[0].NRORDEMSERV} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Categoria
            </h5>
            <p> ${dadosOS[0].NMCATEGORIAMANUT} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Tipo de Manutenção
            </h5>
            <p> ${dadosOS[0].NMTIPOMANUTENCAO} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Prioridade
            </h5>
            <p> ${dadosOS[0].NMPRIORIDADEMANUT} </p>
          </div>
          <div style="margin-right: 4%">
            <h5>
              Data
            </h5>
            <p> ${dadosOS[0].DTORDEMSERV} </p>
          </div>
        </div>

        <div>
          <h5>
            Equipamento
          </h5>
          <p> ${dadosOS[0].NMEQUIPAMENTO} </p>
        </div>

        <div>
          <h5>
            Responsáveis
          </h5>
          <p> ${dadosOS[0].NMFUNCIONARIO} </p>
        </div>

        <div>
          <h5>
            Descrição
          </h5>
          <p> ${dadosOS[0].DSORDEMSERV} </p>
        </div>

      </div>
    `;
      newWindow?.document.write(ordemHTML);
      newWindow?.print();
    });
  }

  public criarCardsOrdensServico() {
    Http.get('/OrdemServ/Equipamento/CategoriaManut/PrioridadeManut/ResponsaveisOrdemServ/Funcionario').then((response: IDictionary) => {
      const dadosOS = response.data.dataset.OrdemServ;

      Metadata.getInstance<IterableComponentRender>('iterable-cards-ordensDeServico').datasource.updateData(dadosOS);
      this.maisQueUmaPagina({
        component: Metadata.getInstance<Button>('proxima-pagina-botao_iterable-cards-ordensDeServico'),
      });
    });
  }

  public criarCardsSolicitacaoServico() {
    Http.get('/SolicitacaoServ/Equipamento/CategoriaManut/Funcionario').then((response: IDictionary) => {
      const dadosSS = response.data.dataset.SolicitacaoServ;

      Metadata.getInstance<IterableComponentRender>('iterable-cards-solicitacoesServico').datasource.updateData(dadosSS);
      this.maisQueUmaPagina({
        component: Metadata.getInstance<Button>('proxima-pagina-botao_iterable-cards-solicitacoesServico'),
      });
    });
  }

  public proximaPagina({ component }: any) {
    component.parent.datasource.page += 1;
    if (component.parent.paginationLength === component.parent.datasource.page) {
      component.disabled = true;
    }
    component.parent.datasource.get();
    Metadata.getInstance<Button>(`pagina-anterior-botao_${component.name.split('_')[1]}`).disabled = false;
  }

  public paginaAnterior({ component }: any) {
    if (component.parent.datasource.page > 1) {
      component.parent.datasource.page -= 1;
      if (component.parent.datasource.page === 1) {
        component.disabled = true;
      }
      component.parent.datasource.get();
      Metadata.getInstance<Button>(`proxima-pagina-botao_${component.name.split('_')[1]}`).disabled = false;
    }
  }

  // Função que desabilita os botões quando tem apenas uma página nos cards
  public maisQueUmaPagina({ component }: any) {
    component.disabled = !(Metadata.getInstance<IterableComponentRender>(component.name.split('_')[1]).paginationLength > 1);
  }

  public filtrarSS({ component }: any) {
    Metadata.getInstance<IterableComponentRender>('iterable-cards-solicitacoesServico').datasource.setSearch(component.value);
  }

  public filtrarOS({ component }: any) {
    Metadata.getInstance<IterableComponentRender>('iterable-cards-ordensDeServico').datasource.setSearch(component.value);
  }
}
